/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004-2007 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.andami.ui.mdiManager;

import java.awt.Rectangle;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;

import org.exolab.castor.xml.XMLException;

import com.iver.andami.PluginServices;
import com.iver.utiles.XMLEntity;


/**
 * This class represents the state of the associated window.
 * The <code>set</code> methods (<code>setX</code>, <code>setY</code>,
 * <code>setHeight</code>, etc) are immediately reflected on the
 * window.
 * 
 * @author Fernando Gonzlez Corts
 */
public class WindowInfo {
	/** Specifies that the window is resizable  */
    public static final int RESIZABLE = 1;

    /** Specifies that the window is maximizable */
    public static final int MAXIMIZABLE = 2;

    /** Specifies that the window is iconifiable */
    public static final int ICONIFIABLE = 4;

    /** Specifies that the window is modal */
    public static final int MODALDIALOG = 8;
    /** Specifies that the window is modeless (it's on the top but doesn't block any other window) */
    public static final int MODELESSDIALOG = 16;
    /** Specifies that the window may be docked inside another window */
    public static final int PALETTE = 32;
    private PropertyChangeSupport support = new PropertyChangeSupport(this);

    /** DOCUMENT ME! */
    private boolean resizable = false;

    /** DOCUMENT ME! */
    private boolean maximizable = false;

    /** DOCUMENT ME! */
    private boolean iconifiable = false;

    /** DOCUMENT ME! */
    private boolean modal = false;
    private boolean modeless = false;

    /**
     * Do we want to persist the geometry of this window in the project file?
     */
    private boolean persistWindow = true;

    /**
     * Se usa para poner una ventana de tipo paleta, por
     * encima de las dems. Equivale a poner usar
     * JDesktopPane.PALETTE_LAYER
     */
    private boolean palette = false;

    /** These properties store the dimension and position of the frame */
    private int width = -1;
    private int height = -1;
    private int x = -1;
    private int y = -1;
    /**
     * These properties store the position and dimension of the frame when it is not maximized
     * (so that it can be restored to its original size). They are equal to the not-normal properties
     * when the frame is not maximized, and different when the frame is maximized.
     */
    private int normalX = 0;
    private int normalY = 0;
    private int normalHeight = -1;
    private int normalWidth = -1;
    /* Whether the window is maximized */
    private boolean isMaximized = false;
    private boolean visible = true;
    /* Whether the window is closed */
    private boolean isClosed = false;


    /** DOCUMENT ME! */
    private String title;

    private int id;

    /**
     * ActionCommand del tool seleccionado. Lo usamos
     * para activar el tool que estaba seleccionado en
     * la vista, cuando volvemos a ella.
     */
    private String selectedTool;

    /**
     * Returns the current x coordinate of the window's origin
     * (left-upper corner of the window).
     *
     * @return Returns the value (in pixels) of the x coordinate
     * of the window's origin.
     */
    public int getX() {
        return x;
    }

    /**
     * Sets the x coordinate of the window's origin
     * (left-upper corner of the window).
     *
     * @param x	The value (in pixels) of the x coordinate to set.
     */
    public void setX(int x) {
    	support.firePropertyChange("x", this.x, x);
    	this.x = x;
    	if (!isMaximized)
    		this.normalX = x;
    }

    /**
     * Gets the value of the y coordinate for the origin
     * (left-upper corner of the window) of the associated
     * window.
     * 
     * @return Returns the y coordinate (in pixels).
     */
    public int getY() {
        return y;
    }

    /**
     * Sets the value of the y coordinate for the origin
     * (left-upper corner of the window) of the associated
     * window.
     * 
     * @param y The value (in pixels) of the y coordinate
     */
    public void setY(int y) {
    	support.firePropertyChange("y", this.y, y);
        this.y = y;
        if (!isMaximized)
        	this.normalY = y;
    }

    /**
     * DOCUMENT ME!
     *
     * @param listener
     */
    public void addPropertyChangeListener(PropertyChangeListener listener) {
        support.addPropertyChangeListener(listener);
    }

    /**
     * DOCUMENT ME!
     *
     * @param listener
     */
    public void removePropertyChangeListener(PropertyChangeListener listener) {
        support.removePropertyChangeListener(listener);
    }

    /**
     * Creates a new WindowInfo object with the provided properties.
     * Valid properties include:
     * <ul>
     * <li>WindowInfo.MODALDIALOG</li>
     * <li>WindowInfo.MODELESSDIALOG</li>
     * <li>WindowInfo.PALETTE</li>
     * <li>WindowInfo.ICONIFIABLE</li>
     * <li>WindowInfo.RESIZABLE</li>
     * <li>WindowInfo.MAXIMIZABLE</li>
     * 
     * <p>Properties can be set together by using the binary OR operator
     * "<b>|</b>". For example:</p>
     * 
     * <p><code>WindowInfo wi = new WindowInfo(WindowInfo.MODELESSDIALOG 
     *   |WindowInfo.ICONIFIABLE|WindowInfo.RESIZABLE).</code></p>
     *   
     * <p>The WindowInfo.MODELESSDIALOG and WindowInfo.MODALDIALOG properties
     * cannot be set at the same time</p>
     * 
     *
     * @param code Bit-or of the window properties.
     *
     * @throws IllegalStateException If incompatible properties are set together,
     * for example, if MODALDIALOG and MODELESSDIALGO are set together.
     */
    public WindowInfo(int code) {
        resizable = (code % 2) > 0;
        code = code / 2;
        maximizable = (code % 2) > 0;
        code = code / 2;
        iconifiable = (code % 2) > 0;
        code = code / 2;
        modal = (code % 2) > 0;
        code = code / 2;
        modeless = (code % 2) > 0;
        code = code / 2;
        palette = (code % 2) > 0;

        if (modal && modeless) {
            throw new IllegalStateException("modal && modeless");
        }
    }
    
    /**
     * Creates a new WindowInfo object with the default properties:
     * <ul>
     * <li>not resizable</li>
     * <li>not maximizable</li>
     * <li>not iconifiable</li>
     * <li>not modal</li>
     * <li>not modeless</li>
     * <li>not palette</li>
     * </uil>
     *
     */
    public WindowInfo(){

    }

    /**
     * Determines whether the associated window is iconifiable or not
     *
     * @return
     */
    public boolean isIconifiable() {
        return iconifiable;
    }

    /**
     * Determines whether the associated window is maximizable or not
     *
     * @return
     */
    public boolean isMaximizable() {
        return maximizable;
    }

    /**
     * Determines whether the associated window is resizable or not
     *
     * @return
     */
    public boolean isResizable() {
        return resizable;
    }

    /**
     * Determines whether the associated window is modal or not
     *
     * @return
     */
    public boolean isModal() {
        return modal;
    }

    /**
     * Gets the window height.
     *
     * @return The window height (in pixels).
     */
    public int getHeight() {
        return height;
    }

    /**
     * Gets the window width.
     *
     * @return The window width (in pixels).
     */
    public int getWidth() {
        return width;
    }

    /**
     * Sets the window height.
     *
     * @param The window height (in pixels)
     */
    public void setHeight(int h) {
    	if (h!=-1)
    		support.firePropertyChange("height", this.height, h);
    	height = h;
        if (!isMaximized)
        	this.normalHeight = h;
    }

    /**
     * Sets the width property for the associated Window.
     *
     * @param w The new width.
     */
    public void setWidth(int w) {
    	if (w!=-1)
    		support.firePropertyChange("width", this.width, w);
        width = w;
        if (!isMaximized)
        	this.width = w;
    }

    /**
     * Gets the title property
     *
     * @return
     */
    public String getTitle() {
        return title;
    }

    /**
     * Sets the title property.
     *
     * @param title The new title.
     */
    public void setTitle(String string) {
    	support.firePropertyChange("title", this.title, string);
        title = string;
    }

    /**
     * Determines whether the associated window is modeless
     * (it is on the top but does not block any other window)
     *
     * @return
     */
    public boolean isModeless() {
        return modeless;
    }

     /**
      * Determines whether the associated window is visible
      *
      * @return true if the associated window is visible, false
      * if it is hidden
      */
    public boolean isVisible() {
        return visible;
    }

    /**
     * Determines whether the associated dockable window is
     * currently in palette status or docked.
     *
     * @return true if the window is currently in palette status,
     * false if it is docked in another window
     */
    public boolean isPalette()
    {
        return palette;
    }

    /**
     * Sets the window's ID.
     * 
     * @param id An integer to identify the window. Different
     * windows must have different IDs.
     */
    public void setId(int id){
    	this.id = id;
    }

    /**
     * Gets the window ID.
     * 
     * @return id An integer to identify the window. Different
     * windows must have different IDs.
     */
    public int getId(){
    	return id;
    }

    /**
     * Updates all the properties in this object so that
     * they match the properties of the provided
     * <code>wi</code> object. The changes are not reflected on the
     * window, just the WindowInfo object is updated.
     * 
     * @param vi A WindowInfo object containing the new
     * properties of the window.
     */
    public void setWindowInfo(WindowInfo vi){
    	this.resizable = vi.resizable;
    	this.maximizable = vi.maximizable;
    	this.isMaximized = vi.isMaximized;
    	this.iconifiable = vi.iconifiable;
    	this.modal = vi.modal;
    	this.modeless = vi.modeless;
    	if (vi.width!=-1)
    		this.width = vi.width;
    	if (vi.height!=-1)
    		this.height = vi.height;
    	this.x = vi.x;
    	this.y = vi.y;
    	this.visible = vi.visible;
    	this.title = vi.title;
    	this.id = vi.id;
    	if (vi.normalHeight!=-1)
    		this.normalHeight = vi.normalHeight;
    	if (vi.normalWidth!=-1)
    		this.normalWidth = vi.normalWidth;
    	this.normalX = vi.normalX;
    	this.normalY = vi.normalY;
    	this.isClosed = vi.isClosed;
    	this.persistWindow = vi.persistWindow;
    }
    
    /**
     * Sets whether the window is in palette mode or is in docked mode. 
     * This method <b>does not</b> update the window status, it just
     * updates the WindowInfo object. Use the IWindowTransform interface
     * to actually change the window status.
     * 
     * @param b
     */
    public void toPalette(boolean b){
    	this.palette=b;
    }

    /**
     * Gets the actionCommand of the currently selected tool for this
     * window.
     * @return
     */
    public String getSelectedTool() {
        return selectedTool;
    }

    /**
     * Sets the actionCommand of the currently selected tool for this
     * window.

     * @param selectedTool The actionCommand of the tool which is
     * going to be selected for this window.
     */
    public void setSelectedTool(String selectedTool) {
        if (selectedTool != null)
            this.selectedTool = selectedTool;
    }

    /**
     *  Finds out whether a window is open (showing) or closed
     */
    public boolean isClosed() {
    	return isClosed;
    }

    /**
     *  Specifies whether a window is open (showing) or closed
     */
    public void setClosed(boolean closed) {
    	support.firePropertyChange("closed", this.isClosed, closed);
    	this.isClosed = closed;
    }


    /**
     * Sets the x coordinate of the window's origin
     * when the window is not maximized. When the window is maximized,
     * it does not change the current window's position, but the
     * window will have this x coordinate when it gets restored.
     *
     * @param normalX	The value (in pixels) of the x coordinate to set.
     */
    public void setNormalX(int normalX) {
    	support.firePropertyChange("normalX", this.normalX, normalX);
    	this.normalX = normalX;
    }

    /**
     * Sets the y coordinate of the window's origin
     * when the window is not maximized. When the window is maximized,
     * it does not change the current window's position, but the
     * window will have this y coordinate when it gets restored.
     *
     * @param normalY	The value (in pixels) of the y coordinate to set.
     */
    public void setNormalY(int normalY) {
    	support.firePropertyChange("normalY", this.normalY, normalY);
    	this.normalY = normalY;
    }

    /**
     * Sets the height of the window
     * when the window is not maximized. When the window is maximized,
     * it does not change the current window's height, but the
     * window will have this height when it gets restored.
     *
     * @param normalHeight	The height (in pixels) to set.
     */
    public void setNormalHeight(int normalHeight) {
    	support.firePropertyChange("normalHeight", this.normalHeight, normalHeight);
    	this.normalHeight = normalHeight;
    }

    /**
     * Sets the width of the window
     * when the window is not maximized. When the window is maximized,
     * it does not change the current window's width, but the
     * window will have this width when it gets restored.
     *
     * @param normalWidth	The width (in pixels) to set.
     */
    public void setNormalWidth(int normalWidth) {
    	support.firePropertyChange("normalWidth", this.normalWidth, normalWidth);
    	this.normalWidth = normalWidth;
    }

    /**
     * Specifies whether the window is maximized or not.
     */ 
    public void setMaximized(boolean maximized) {
    	support.firePropertyChange("maximized", this.isMaximized, maximized);
    	this.isMaximized = maximized;
    }

    /**
     * Specifies whether the window is maximizable or not.
     */ 
    public void setMaximizable(boolean maximizable) {
    	this.maximizable = maximizable;
    }

    private void setBounds(int x, int y, int width, int height) {
    	support.firePropertyChange("x", this.height, x);
    	this.x = x;
    	support.firePropertyChange("y", this.height, y);
    	this.y = y;
    	support.firePropertyChange("width", this.width, width);
    	this.width = width;
    	support.firePropertyChange("height", this.height, height);
    	this.height = height;
    }
    
    /**
     * Sets the bounds of the window
     * when the window is not maximized. When the window is maximized,
     * it does not change the current window's bounds, but the
     * window will have this bounds when it gets restored.
     *
     * @param x	The normal x (in pixels) to set.
     * @param y	The normal y (in pixels) to set.
     * @param width	The normal width (in pixels) to set.
     * @param height	The normal height (in pixels) to set.
     */
    public void setNormalBounds(int x, int y, int width, int height) {
    	support.firePropertyChange("normalX", this.normalX, x);
    	this.normalX = x;
    	support.firePropertyChange("normalY", this.normalY, y);
    	this.normalY = y;
    	support.firePropertyChange("normalWidth", this.normalWidth, width);
    	this.normalWidth = width;
    	support.firePropertyChange("normalWeight", this.normalHeight, height);
    	this.normalHeight = height;
    }
    
    /**
     * Gets the bounds of the window
     * when the window is not maximized. When the window is maximized,
     * it gets the bounds that the window will have when it gets restored.
     * 
     * @return the bounds of the window when the window is not maximized,
     * or if it is maximized, the bounds that the window will have when
     * it gets restored
     */
    public Rectangle getNormalBounds() {
    	return new Rectangle(getNormalX(), getNormalY(), getNormalWidth(), getNormalHeight());
    }

    /**
     * Sets the bounds of the window
     * when the window is not maximized. When the window is maximized,
     * it does not change the current window's bounds, but the
     * window will have this bounds when it gets restored.
     *
     * @param normalBounds The normal bounds to set
     */
    public void setNormalBounds(Rectangle normalBounds) {
    	Rectangle oldBounds = new Rectangle(this.normalX, this.normalY,
    			this.normalWidth, this.normalHeight);
    	support.firePropertyChange("normalBounds", oldBounds, normalBounds);
    	support.firePropertyChange("normalX", this.normalX, normalBounds.x);
    	normalX = normalBounds.x;
    	support.firePropertyChange("normalY", this.normalY, normalBounds.y);
    	normalY = normalBounds.y;
    	support.firePropertyChange("normalHeight", this.normalHeight, normalBounds.height);
    	normalHeight = normalBounds.height;
    	support.firePropertyChange("normalWidth", this.normalWidth, normalBounds.width);
    	normalWidth = normalBounds.width;
    }

    /**
     * Set the bounds of the window.
     * 
     * @param bounds The bounds to set.
     */
    public void setBounds(Rectangle bounds) {
    	support.firePropertyChange("x", this.x, bounds.x);
    	x = bounds.x;
    	support.firePropertyChange("y", this.y, bounds.y);
    	y = bounds.y;
    	support.firePropertyChange("height", this.height, bounds.height);
    	height = bounds.height;
    	support.firePropertyChange("width", this.width, bounds.width);
    	width = bounds.width;
    }

    /**
     * Gets the bounds of the window.
     * 
     * @return the bounds of the window.
     */
    public Rectangle getBounds() {
    	return new Rectangle(x, y, width, height);
    }

    /**
     * Gets the x coordinate of the window's origin when the window is not
     * maximized. When the window is maximized, gets the x coordinate that
     * it will have when it gets restored
     * 
     * @return the normal x coordinate of the window's origin
     */
    public int getNormalX() {
    	if (normalX!=0)
    		return this.normalX;
    	else
    		return x;
    }

    /**
     * Gets the x coordinate of the window's origin when the window is not
     * maximized. When the window is maximized, gets the y coordinate that
     * it will have when it gets restored
     * 
     * @return the normal y coordinate of the window's origin
     */
    public int getNormalY() {
    	if (normalY!=0)
    		return this.normalY;
    	else
    		return y;
    }

    /**
     * Gets the height of the window's origin when the window is not
     * maximized. When the window is maximized, gets the height that
     * it will have when it gets restored
     * 
     * @return the normal height of the window
     */
    public int getNormalHeight() {
    	if (normalHeight!=0)
    		return this.normalHeight;
    	else
    		return height;
    }

    /**
     * Gets the width of the window's origin when the window is not
     * maximized. When the window is maximized, gets the width that
     * it will have when it gets restored
     * 
     * @return the normal width of the window
     */
    public int getNormalWidth() {
    	if (normalWidth!=0)
    		return this.normalWidth;
    	else
    		return width;
    }

    /**
     * Determines whether the window is maximized or not
     * 
     * @return true if the window is maximized, false otherwise
     */
    public boolean isMaximized() {
    	return this.isMaximized;
    }

    /**
     * Checks if the geometry of this window should be persisted in the
     * project file. This is set by the persistWindow(boolean) method,
     * and the default value is true.
     *
     * @return True if the geometry of this window should be persisted
     * in the project files, false otherwise.
     */
    public boolean checkPersistence() {
    	return this.persistWindow;
    }

    /**
     * Set whether the geometry of this window should be persisted in the
     * project files.
     *
     * @param persist
     */
    public void setPersistence(boolean persist) {
    	this.persistWindow = persist;
    }

    /**
	 * Gets the window properties in an XMLEntity object, suitable
	 * for persistence.
	 *
	 * @return An XMLEntity object containing the window properties
     * @throws SaveException
	 * @throws XMLException
	 */
	public XMLEntity getXMLEntity() {
		XMLEntity xml = new XMLEntity();
		xml.setName("ViewInfoProperties");
		xml.putProperty("X", this.getX());
		xml.putProperty("Y", this.getY());
		xml.putProperty("Width", this.getWidth());
		xml.putProperty("Height", this.getHeight());
		xml.putProperty("isVisible", this.isVisible());
		xml.putProperty("isResizable", this.isResizable());
		xml.putProperty("isMaximizable", this.isMaximizable());
		xml.putProperty("isModal", this.isModal());
		xml.putProperty("isModeless", this.isModeless());
		xml.putProperty("isClosed", this.isClosed());
		if (this.isMaximized()==true) {
			xml.putProperty("isMaximized", this.isMaximized());
			xml.putProperty("normalX", this.getNormalX());
			xml.putProperty("normalY", this.getNormalY());
			xml.putProperty("normalWidth", this.getNormalWidth());
			xml.putProperty("normalHeight", this.getNormalHeight());
		}
		return xml;
	}

	/**
	 * Creates a WindowInfo object from an XMLEntity containing the
	 * window properties.
	 *  
	 * @param xml An XMLEntity object containing the window properties
	 * 
	 * @return A new WindowInfo object, containing the properties
	 * specified in the XMLEntity parameters 
	 */
	public static WindowInfo createFromXMLEntity(XMLEntity xml)
	{
		WindowInfo result = new WindowInfo();
		try {
			result.setX(xml.getIntProperty("X"));
			result.setY(xml.getIntProperty("Y"));
			result.setHeight(xml.getIntProperty("Height"));
			result.setWidth(xml.getIntProperty("Width"));
			result.setClosed(xml.getBooleanProperty("isClosed"));
			if (xml.contains("isMaximized")) {
				boolean maximized = xml.getBooleanProperty("isMaximized");
				result.setMaximized(maximized);
				if (maximized==true) {
					result.setNormalBounds(xml.getIntProperty("normalX"), xml.getIntProperty("normalY"), xml.getIntProperty("normalWidth"), xml.getIntProperty("normalHeight"));
				}
				else {
					result.setNormalBounds(result.getBounds());
				}
			}
		}
		catch (com.iver.utiles.NotExistInXMLEntity ex) {
			PluginServices.getLogger().warn(PluginServices.getText(null, "Window_properties_not_stored_correctly_Window_state_will_not_be_restored"));
		}

		return result;
	}

	/**
	 * Updates this WindowInfo object according to the properties
	 * provided by the XMLEntity parameter.
	 *  
	 * @param xml An XMLEntity object containing the window properties
	 */
	public void getPropertiesFromXMLEntity(XMLEntity xml)
	{
		this.x = xml.getIntProperty("X");
		this.y = xml.getIntProperty("Y");
		this.height = xml.getIntProperty("Height");
		this.width = xml.getIntProperty("Width");
		this.isClosed = xml.getBooleanProperty("isClosed");
		if (xml.contains("isMaximized")) {
			boolean maximized = xml.getBooleanProperty("isMaximized");
			this.isMaximized = maximized;
			if (maximized==true) {
				this.setNormalBounds(xml.getIntProperty("normalX"), xml.getIntProperty("normalY"), xml.getIntProperty("normalWidth"), xml.getIntProperty("normalHeight"));
			}
		}
	}
}
