/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.gvsig.rasterTools.pansharpening.filter;

import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.awt.image.PixelGrabber;

import org.cresques.util.ColorSpaceConversion;
import org.cresques.geo.ViewPortData;
import org.cresques.io.GeoRasterFile;
import org.cresques.io.data.RasterBuf;
import org.cresques.px.PxRaster;


/**
 * Filtro de sharpening que se aplica sobre la vista. Tiene como entradas el objeto Image
 * con las bandas del raster visualizadas en la Vista y la banda pancromtica. A partir de
 * estas entradas genera un Image de salida como la que hay pero de resolucin igual a la
 * de la pancromtica.  
 * @author Nacho Brodin (brodin_ign@gva.es)
 *
 */
public class PanSharpeningImageFilter extends PanSharpeningFilter {
	
	//Variable que dice para cada pixel de la imagen de entrada cuantos de salida hay.
	private int 			nOutputPixelsPerInputPixel;  
	//Vector con los desplazamientos de los pixeles de salida dentro del vector
	private int[] 			outKernel;
	//Kernel aplicado a la pancromatica 
	private Kernel			kernel = null; 
	private PixelGrabber 	pgAct = null, pgPrev = null, pgNext = null;
	int[]					bufferPrev = null, bufferAct = null, bufferNext = null;
	
    /**
     * Constructor
     *
     */
    public PanSharpeningImageFilter() {
        super();
    }

    /**
     *  Validamos que haya alguna otra banda adems de la pancromtica y que la pancromtica 
     *  sea de mayor resolucin que las dems.
     */
    private void checkInput(){
    	exec = true;
        if(heightMultiespec >= heightPancr || widthMultiespec >= widthPancr || heightMultiespec == 0 || widthMultiespec == 0)
			exec = false;
        for(int i=0;i<files.length;i++){
        	if(i != posPancromatica){
        		if(files[i].getHeight() != heightMultiespec || files[i].getWidth() != widthMultiespec){
        			exec = false;
        			break;
        		}   		
        	}
        }
    }
    
    /**
     * Carga los parmetros pasados al filtro en las variables de 
     * instancia que corresponde,
     */
    private void loadParam(){
    	//Carga de parametros
    	this.image = (Image) params.get("raster");
        posPancromatica = ((Integer) params.get("pancromatica")).intValue();
        files = (GeoRasterFile[]) params.get("files");
        bandOrder = (int[]) params.get("order");
        alpha = ((Integer) params.get("alpha")).intValue();
        method = (String) params.get("method");
        coef = ((Double) params.get("coef")).doubleValue();
        coefBrovey = ((Integer) params.get("coefBrovey")).intValue();
        
        //Asignamos el nombre de la banda pancromtica como parmetro para que cuando 
        //volvamos a abrir el dialogo pueda recuperarse y seleccionarse en el cuadro
        pancrName = files[posPancromatica].getName().substring(
        				files[posPancromatica].getName().lastIndexOf("/")+1, 
        				files[posPancromatica].getName().length());
        this.addParam("pancrName", pancrName);
       
        height = image.getHeight(null);
        width = image.getWidth(null);
        heightPancr = files[posPancromatica].getHeight();
        widthPancr = files[posPancromatica].getWidth();
        for(int i=0;i<files.length;i++){
        	if(i != posPancromatica){
        		heightMultiespec = files[i].getHeight(); 
        		widthMultiespec = files[i].getWidth();
        	}
        }
        
        relX = (int)widthPancr/widthMultiespec;
		relY = (int)heightPancr/heightMultiespec;
    }
    
    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#pre()
     */
    public void pre() {
    	
    	loadParam();
    	checkInput();
    	
        //Creamos el buffer donde se pinta la pancromatica
        
    	imagePancr = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
        Graphics2D graphicsPancr = (Graphics2D)imagePancr.getGraphics();
		PxRaster rasterPancr = new PxRaster(files[posPancromatica], null, files[posPancromatica].getView());
						
		rasterPancr.setBand(GeoRasterFile.RED_BAND, 0);
		rasterPancr.setBand(GeoRasterFile.GREEN_BAND, 1);
		rasterPancr.setBand(GeoRasterFile.BLUE_BAND, 2);
		
		ViewPortData vp = (ViewPortData)viewPortData.clone();
		vp.zoom(extent);
		rasterPancr.draw(graphicsPancr, vp);
		rasterPancr = null;
		
		//Creamos el buffer donde se pinta la imagen de entrada
		
		imageMultiespec = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
	    Graphics2D graphicsMultiespec = (Graphics2D)imageMultiespec.getGraphics();
		PxRaster rasterMultiespec = null;
		boolean first = true;
		for(int i = 0;i<files.length;i++){	
		  if(first){
			rasterMultiespec = new PxRaster(files[i], null, files[posPancromatica].getView());
			first = false;
		  }else
		   	rasterMultiespec.addFile(files[i].getName());
		}
		
		rasterMultiespec.setBand(GeoRasterFile.RED_BAND, bandOrder[0]);
		rasterMultiespec.setBand(GeoRasterFile.GREEN_BAND, bandOrder[1]);
		rasterMultiespec.setBand(GeoRasterFile.BLUE_BAND, bandOrder[2]);
							
		rasterMultiespec.draw(graphicsMultiespec, vp);
		rasterMultiespec = null;
		
        super.pre();
    }

    /**
     * Aplica la operacin de refinamiento sobre el buffer Multiespectral que contiene 
     * el RGB que es pasado por parmetro utilizando la pancromtica.
     * @param bufferInput	Buffer rgb 
     * @param length		longitud del buffer de la pancromatica utilizado
     * @param iLine			lnea leida de la imagen multiespectral
     * @return				buffer con el resultado de la operacin
     */
    private int[] processBrovey(int[] bufferInput, int iLine){    	
    	double[]		hsl;
    	int[]			rgb;    	
    	PixelGrabber 	pg = null;
    	
    	//Longitud del buffer de salida
    	int 			widthDst = width * relX;
    	//Buffer de salida
    	int[]			bufferPancr = new int[width];

    	pg = new PixelGrabber(imagePancr, 0, iLine, width, 1, bufferPancr , 0, width);

    	
    	try {
            pg.grabPixels();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }
   
        for(int iElem=0; iElem<width; iElem++){
    		int r = ((bufferInput[iElem] >> 16) & 0xff);
			int g = ((bufferInput[iElem] >> 8) & 0xff);
			int b = (bufferInput[iElem] & 0xff);
			byte i = (byte)((bufferPancr[iElem] >> 16) & 0xff);
			double scale = (3.0*(i+coefBrovey))/(r+g+b+1.0);
			r *= scale;g *= scale;b *= scale;			
			bufferPancr[iElem] = ((alpha << 24) & 0xff000000) | 
								((r << 16) & 0x00ff0000)| 
								((g << 8) & 0x0000ff00) | 
								(b & 0x000000ff);
    	}
    	return bufferPancr;
    	
    }
    
    /**
     * Aplica la operacin de refinamiento sobre el buffer Multiespectral que contiene 
     * el RGB que es pasado por parmetro utilizando la pancromtica.
     * @param bufferInput	Buffer rgb 
     * @param length		longitud del buffer de la pancromatica utilizado
     * @param iLine			lnea leida de la imagen multiespectral
     * @return				buffer con el resultado de la operacin
     */
    private int[] processKernel(int[] bufferInput, int iLine){    	
    	int[]			bufferDst = new int[width];
    	    	
    	if(iLine != 0){
    		bufferPrev = bufferAct;
    		bufferAct = bufferNext;
    	}else{
    		bufferPrev = null;
    		bufferAct = new int[width];
    		pgAct = new PixelGrabber(imagePancr, 0, iLine, width, 1, bufferAct , 0, width);
    		try {
                pgAct.grabPixels();
            } catch (InterruptedException e) {e.printStackTrace();}
    	}
    	    	
    	if(iLine != (height - 1)){
    		bufferNext = new int[width];
    		pgNext = new PixelGrabber(imagePancr, 0, iLine + 1, width, 1, bufferNext , 0, width);
    		try {
                pgNext.grabPixels();
            } catch (InterruptedException e) {e.printStackTrace();}
    	}else
    		bufferNext = null;
    	        

    	for(int iElem=0; iElem<width; iElem++){
    		int a = ((bufferInput[iElem] >> 24) & 0xff);
    		int r = ((bufferInput[iElem] >> 16) & 0xff);
			int g = ((bufferInput[iElem] >> 8) & 0xff);
			int b = (bufferInput[iElem] & 0xff);

			double[][] op = new double[3][3];
			if(bufferPrev != null){
				if(iElem != 0)op[0][0] = (bufferPrev[iElem - 1] & 0x000000ff);
				op[0][1] = (bufferPrev[iElem] & 0x000000ff);
				if(iElem != (width -1))op[0][2] = (bufferPrev[iElem + 1] & 0x000000ff);
			}
			if(iElem != 0)op[1][0] = (bufferAct[iElem - 1] & 0x000000ff);
			op[1][1] = (bufferAct[iElem] & 0x000000ff);
			if(iElem != (width -1))op[1][2] = (bufferAct[iElem + 1] & 0x000000ff);
			if(bufferNext != null){
				if(iElem != 0)op[2][0] = (bufferNext[iElem - 1] & 0x000000ff); 
				op[2][1] = (bufferNext[iElem] & 0x000000ff);
				if(iElem != (width -1))op[2][2] = (bufferNext[iElem + 1] & 0x000000ff);
			}
			Kernel operando = new Kernel(op);
			double i = kernel.kernelOperation(operando) * 0.15;
			r += i;
			g += i;
			b += i;
			bufferDst[iElem] = ((a << 24) & 0xff000000) | 
								((r << 16) & 0x00ff0000)| 
								((g << 8) & 0x0000ff00) | 
								(b & 0x000000ff);	
    	}
    	return bufferDst;
    	
    }
    
    /**
     * Aplica la operacin de refinamiento sobre el buffer Multiespectral que contiene 
     * el RGB que es pasado por parmetro utilizando la pancromtica.
     * @param bufferInput	Buffer rgb 
     * @param length		longitud del buffer de la pancromatica utilizado
     * @param iLine			lnea leida de la imagen multiespectral
     * @return				buffer con el resultado de la operacin
     */
    private int[] processIHS(int[] bufferInput, int iLine){    	
    	double[]		hsl;
    	int[]			rgb;
    	PixelGrabber 	pg = null;

    	//Buffer de salida
    	int[]			bufferPancr = new int[width];

    	pg = new PixelGrabber(imagePancr, 0, iLine, width, 1, bufferPancr , 0, width);

    	try {
            pg.grabPixels();
        } catch (InterruptedException e) {
            e.printStackTrace();
        }

        int[] uvw , tmp = new int[3];
        double[] xyz;
    	for(int iElem=0; iElem<width; iElem++){
    		xyz = ColorSpaceConversion.RGBtoHSL( (bufferInput[iElem] >> 16) & 0x000000ff,
    											 (bufferInput[iElem] >> 8) & 0x000000ff,
    											 bufferInput[iElem] & 0x000000ff);

    		xyz[2] = ((bufferPancr[iElem] & 0x000000ff)/255.0) + coef;
    		
    		tmp[0] = (int)(255.0 * xyz[0] / 360.0 + 0.5);
			tmp[2] = (int) (xyz[2]*255. + 0.5);
			tmp[1] = (int) (xyz[1]*255. + 0.5);
			
    		uvw = ColorSpaceConversion.HSLtoRGB(tmp[ColorSpaceConversion.H],
    											tmp[ColorSpaceConversion.S],
    											tmp[ColorSpaceConversion.L]);
    		bufferPancr[iElem] = ((alpha << 24) & 0xff000000) |
								((uvw[0] << 16) & 0x00ff0000)|
								((uvw[1] << 8) & 0x0000ff00) |
								(uvw[2] & 0x000000ff);
    	}
    	return bufferPancr;
    	
    }
        
    /**
     * Aplica el filtro sobre el raster pasado pixel a pixel
     */
    public void execute() {
        pre();
        if (exec) {    		
        	int[] 			pRGBArrayMultiesp = new int[width];
        	int[] 			pRGBArrayPancr = null;
        	PixelGrabber	pg = null;
        	int 			widthDst = width * relX;
        	
        	//Para cada linea leemos los valores RGB del image. Aplicamos el algoritmo
        	//y escribimos el resultado.
        	
        	if(method.equals("hsl")){
        		double[][]	k = {{-1F, -1F, -1F},{-1F, 8F, -1F},{-1F, -1F, -1F}};
        		kernel = new Kernel(k);
        		for(int iLine=0;iLine<height ;iLine++){
		    		pg = new PixelGrabber(imageMultiespec, 0, iLine, width, 1, pRGBArrayMultiesp, 0, width);
		    		try {
		                pg.grabPixels();
		            } catch (InterruptedException e) {e.printStackTrace();}    
		            pRGBArrayPancr = processIHS(pRGBArrayMultiesp, iLine);	    		
		    		((BufferedImage)imagePancr).setRGB(0, iLine, width, 1, pRGBArrayPancr, 0, width);
		    	}
        	}else{
		    	for(int iLine=0;iLine<height ;iLine++){
		    		pg = new PixelGrabber(imageMultiespec, 0, iLine, width, 1, pRGBArrayMultiesp, 0, width);
		    		try {
		                pg.grabPixels();
		            } catch (InterruptedException e) {e.printStackTrace();}    
		            pRGBArrayPancr = processBrovey(pRGBArrayMultiesp, iLine);	    		
		    		((BufferedImage)imagePancr).setRGB(0, iLine, width, 1, pRGBArrayPancr, 0, width);
		    	}
        	}
        }
        
        post();
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#process(int, int)
     */
    public void process(int x, int y) {

    }
    
    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#processSuperSampling(int, int)
     */
    public void processSuperSampling(int x, int y) {
    
	}

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getInRasterDataType()
     */
    public int getInRasterDataType() {
        return RasterBuf.TYPE_IMAGE;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getOutRasterDataType()
     */
    public int getOutRasterDataType() {
        return RasterBuf.TYPE_IMAGE;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#getResult(java.lang.String)
     */
    public Object getResult(String name) {
        if (name.equals("raster")) {
            return (Object) this.imagePancr;
        } else {
            return null;
        }
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.RasterFilter#processLine(int)
     */
    public void processLine(int y) {
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRasterFilter#post()
     */
    public void post() {
    }
}

