/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io;

import java.awt.Image;
import java.awt.geom.NoninvertibleTransformException;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.io.File;

import org.cresques.cts.ICoordTrans;
import org.cresques.cts.IProjection;
import org.cresques.io.data.BandList;
import org.cresques.io.data.RasterBuf;
import org.cresques.io.datastruct.Metadata;
import org.cresques.io.exceptions.NotSupportedExtensionException;
import org.cresques.io.exceptions.SupersamplingNotSupportedException;
import org.cresques.px.Extent;
import org.cresques.util.Utilities;

import es.gva.cit.jgdal.GdalException;

/**
 * @author Luis W. Sevilla
 */
public class GdalFile extends GeoRasterFile {
	public static void seflRegister(){

	}

	public final static int 	BAND_HEIGHT = 64;
	protected GdalNative 		file = null;
	/**
	 * Tama�o de pixel para las imagenes con fichero RMF. No podemos salvarlo en file porque es necesario conocer el
	 * tama�o de pixel asignado por rl .rmf y el tama�o de pixel real.
	 */
	private double				pixelSizeX = 0D, pixelSizeY = 0D;

	private Extent v = null;

	public GdalFile(IProjection proj, String fName, boolean hdf){
		super(proj, fName);
	}

	public GdalFile(IProjection proj, String fName)throws NotSupportedExtensionException{
		super(proj, fName);
		extent = new Extent();
		fName = translateFileName(fName);
		try {
			file = new GdalNative(fName, this);
			load();
			readGeoInfo(fName);
			bandCount = file.getRasterCount();
			if ( bandCount > 2) {
				setBand(RED_BAND,   0);
				setBand(GREEN_BAND, 1);
				setBand(BLUE_BAND,  2);
			} else
				setBand(RED_BAND|GREEN_BAND|BLUE_BAND, 0);
		} catch (GdalException e) {
			throw new NotSupportedExtensionException("Extension not supported");
		} catch(Exception e){
		  	System.out.println("Error en GdalOpen");
		  	e.printStackTrace();
		  	file = null;
		}

		//Obtenemos el tipo de dato de gdal y lo convertimos el de RasterBuf
		setDataType(org.cresques.util.Utilities.getRasterBufTypeFromGdalType(file.getDataType()));
	}

	public String translateFileName(String fileName){
		String ext = fileName.substring(fileName.lastIndexOf(".")+1);
		if (ext.toLowerCase().equals("hdr")){
			String path = fileName.substring(0, fileName.lastIndexOf("."));
			File file = new File(path);
			if (file.exists()){
				return path;
			}else{
				file = new File(path + ".dat");
				if (file.exists()){
					return path + ".dat";
				}
			}
			return null;
		}
		return fileName;
	}

	/**
	 * Obtenemos o calculamos el extent de la imagen.
	 */
	public GeoFile load() {
		extent = new Extent(file.bBoxRot.minX, file.bBoxRot.minY, file.bBoxRot.maxX, file.bBoxRot.maxY);
		requestExtent = new Extent(file.bBoxWithoutRot.minX, file.bBoxWithoutRot.minY, file.bBoxWithoutRot.maxX, file.bBoxWithoutRot.maxY);
		return this;
	}

	/**
	 * Cierra el fichero de imagen
	 */
	public void close() {
		try {
			if(file != null){
				file.close();
				file = null;
			}
		} catch (GdalException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * Asigna a cada banda R,G o B una banda de la imagen
	 */
	public void setBand(int flag, int bandNr) {
		super.setBand(flag, bandNr);
		if ((flag & GeoRasterFile.RED_BAND) == GeoRasterFile.RED_BAND) file.rBandNr = bandNr+1;
		if ((flag & GeoRasterFile.GREEN_BAND) == GeoRasterFile.GREEN_BAND) file.gBandNr = bandNr+1;
		if ((flag & GeoRasterFile.BLUE_BAND) == GeoRasterFile.BLUE_BAND) file.bBandNr = bandNr+1;
	}

	/**
	 * Asigna el extent de la vista actual. existe un fichero .rmf debemos hacer una transformaci�n
	 * de la vista asignada ya que la petici�n viene en coordenadas del fichero .rmf y la vista (v)
	 * ha de estar en coordenadas del fichero.
	 */
	public void setView(Extent e) {
		if(rmfExists){

			Point2D.Double petInit = null, petEnd = null;
			try{
				petInit = new Point2D.Double(e.minX(),  e.maxY());
				petEnd = new Point2D.Double(e.maxX(), e.minY());
				transformRMF.inverseTransform(petInit, petInit);
				transformRMF.inverseTransform(petEnd, petEnd);
				transformTFW.transform(petInit, petInit);
				transformTFW.transform(petEnd, petEnd);
			}catch(NoninvertibleTransformException ex){}
			double h = file.bBoxWithoutRot.maxY - file.bBoxWithoutRot.minY;
			if(!file.isGeoreferenced())
				v = new Extent(	petInit.getX(), h - petInit.getY(), petEnd.getX(), h - petEnd.getY());
			else
				v = new Extent(	petInit.getX(), petInit.getY(), petEnd.getX(), petEnd.getY());

		}else
			v = new Extent(e.minX(), e.minY(), e.maxX(), e.maxY());
	}

	 /**
	 * Calcula la transformaci�n que se produce sobre la vista cuando la imagen tiene un fichero .rmf
	 * asociado. En Gdal el origen de coordenadas en Y es el valor m�nimo y crece hasta el m�ximo. De la
	 * misma forma calcula la matriz de transformaci�n de la cabecera del fichero o del world file asociado
	 * @param originX Origen de la imagen en la coordenada X
	 * @param originY Origen de la imagen en la coordenada Y
	 */
	public void setExtentTransform(double originX, double originY, double psX, double psY) {
		transformRMF.setToTranslation(originX, originY);
		transformRMF.scale(psX, psY);

		if(file.trans != null){
			transformTFW.setToTranslation(file.trans.adfgeotransform[0], file.trans.adfgeotransform[3]);
			transformTFW.scale(file.trans.adfgeotransform[1], file.trans.adfgeotransform[5]);
		}
	}

	/**
	 * Obtiene extent de la vista actual
	 */
	public Extent getView() {
		return v;
	}

	/**
	 * Obtiene la anchura del fichero
	 */
	public int getWidth() {
		return file.width;
	}

	/**
	 * Obtiene la altura del fichero
	 */
	public int getHeight() {
		return file.height;
	}

	/* (non-Javadoc)
	 * @see org.cresques.io.GeoRasterFile#reProject(org.cresques.cts.ICoordTrans)
	 */
	public void reProject(ICoordTrans rp) {
		// TODO Auto-generated method stub
	}

	/**
	 * Obtiene la orientaci�n de la imagen a partir del signo del tama�o de pixel para poder
	 * asignarlo en el setView. Esto es util para poder conocer como debe leerse la image,
	 * de abajo a arriba, de arriba a abajo, de izquierda a derecha o de derecha a izquierda.
	 * La posici�n habitual es la que el pixel size en X es positivo y en Y negativo leyendose
	 * en este caso las X de menor a mayor y las Y de mayor a menor. Los casos posibles son:
	 * <UL>
	 * <LI><B>X > 0; Y < 0;</B> {true, false}</LI>
	 * <LI><B>X > 0; Y > 0;</B> {true, true}</LI>
	 * <LI><B>X < 0; Y > 0;</B> {false, true}</LI>
	 * <LI><B>X < 0; Y < 0;</B> {false, false}</LI>
	 * </UL>
	 *
	 * @return
	 */
	private boolean[] getOrientation(){
		boolean[] orientation = {true, false};
		if(!rmfExists){
			if(file.trans != null && file.trans.adfgeotransform != null && file.trans.adfgeotransform[5] > 0)
				orientation[1] = true;
			if(file.trans != null && file.trans.adfgeotransform != null && file.trans.adfgeotransform[1] < 0)
				orientation[0] = false;
		}else{
			if(rmfTransform.getScaleY() > 0)
				orientation[1] = true;
			if(rmfTransform.getScaleX() < 0)
				orientation[0] = false;
		}
		return orientation;
	}

	/* (non-Javadoc)
	 * @see org.cresques.io.GeoRasterFile#updateImage(int, int, org.cresques.cts.ICoordTrans)
	 */
	public Image updateImage(int width, int height, ICoordTrans rp) {
		int line, pRGBArray[] = null;
		Image image = null;

		if (mustVerifySize()) {
			// Work out the correct aspect for the setView call.
			double dFileAspect = (double)v.width()/(double)v.height();
			double dWindowAspect = (double)width /(double)height;

			if (dFileAspect > dWindowAspect) {
			  height =(int)((double)width/dFileAspect);
			} else {
			  width = (int)((double)height*dFileAspect);
			}
		}

		// Set the view
		file.setView(v.minX(), v.maxY(), v.maxX(), v.minY(), width, height, getOrientation());
		setStep(file.stepArrayX, file.stepArrayY);

		if(width<=0)width=1;
		if(height<=0)height=1;

		image = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
		//image = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
		pRGBArray = new int[width/**BAND_HEIGHT*/];
		try {
			//int nLin = height % BAND_HEIGHT;
			file.setAlpha(getAlpha());
			setBand(RED_BAND,   rBandNr);
			setBand(GREEN_BAND, gBandNr);
			setBand(BLUE_BAND,  bBandNr);
			for (line=0; line < height; line++) { //+=BAND_HEIGHT) {
				//int bandH = Math.min(BAND_HEIGHT, height-line);
				//file.readBandRGBA(bandH, BAND_HEIGHT, pRGBArray);
				file.readLineRGBA(pRGBArray);
				setRGBLine((BufferedImage) image, 0, line, width, 1/*bandH*/, pRGBArray, 0, width);
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return image;
	}

	public RasterBuf getRaster(int width, int height, ICoordTrans rp) {
		int line;
		RasterBuf raster = null;

		if(mustVerifySize()){
			// Work out the correct aspect for the setView call.
			double dFileAspect = (double)v.width()/(double)v.height();
			double dWindowAspect = (double)width /(double)height;

			if (dFileAspect > dWindowAspect) {
			  height =(int)((double)width/dFileAspect);
			} else {
			  width = (int)((double)height*dFileAspect);
			}
		}

		// Set the view
		boolean[] orientation = getOrientation();
		file.setView(v.minX(), v.maxY(), v.maxX(), v.minY(),
			width, height, orientation);
		setStep(file.stepArrayX, file.stepArrayY);

		try {
			//Esta funci�n se usa para la renderizaci�n, por eso se crean 4 bandas a pi�on fijo
			raster = new RasterBuf(getDataType(), width, height, 4, true);

			file.setAlpha(getAlpha());
			setBand(RED_BAND,   rBandNr);
			setBand(GREEN_BAND, gBandNr);
			setBand(BLUE_BAND,  bBandNr);

			switch(getDataType()){
			case RasterBuf.TYPE_BYTE:
				for (line = 0; line < height; line++)
					file.readLine(raster.getLineByte(line));
				break;
			case RasterBuf.TYPE_SHORT:;
				for (line = 0; line < height; line++)
					file.readLine(raster.getLineShort(line));
				break;
			case RasterBuf.TYPE_INT:
				for (line = 0; line < height; line++)
					file.readLine(raster.getLineInt(line));
				break;
			case RasterBuf.TYPE_FLOAT:
				for (line = 0; line < height; line++)
					file.readLine(raster.getLineFloat(line));
				break;
			case RasterBuf.TYPE_DOUBLE:
				for (line = 0; line < height; line++)
					file.readLine(raster.getLineDouble(line));
				break;
			case RasterBuf.TYPE_UNDEFINED:break;
			}

		} catch (Exception e) {
			e.printStackTrace();
		}

		return raster;
	}

	/**
	 * Asigna al objeto Image los valores con los dato de la imagen contenidos en el
	 * vector de enteros.
	 * @param image	imagen con los datos actuales
	 * @param startX	inicio de la posici�n en X dentro de la imagen
	 * @param startY	inicio de la posici�n en X dentro de la imagen
	 * @param w	Ancho de la imagen
	 * @param h	Alto de la imagen
	 * @param rgbArray	vector que contiene la banda que se va a sustituir
	 * @param offset	desplazamiento
	 * @param scansize	tama�o de imagen recorrida por cada p
	 */
	protected void setRGBLine(BufferedImage image, int startX, int startY, int w, int h, int[] rgbArray,
			 int offset, int scansize) {
		image.setRGB(startX, startY, w, h, rgbArray, offset, scansize);
	}

	/**
	 * Asigna al objeto Image la mezcla entre los valores que ya tiene y los valores
	 * con los dato de la imagen contenidos en el vector de enteros. De los valores RGB
	 * que ya contiene se mantienen las bandas que no coinciden con el valor de flags. La
	 * banda correspondiente a flags es sustituida por los datos del vector.
	 * @param image	imagen con los datos actuales
	 * @param startX	inicio de la posici�n en X dentro de la imagen
	 * @param startY	inicio de la posici�n en X dentro de la imagen
	 * @param w	Ancho de la imagen
	 * @param h	Alto de la imagen
	 * @param rgbArray	vector que contiene la banda que se va a sustituir
	 * @param offset	desplazamiento
	 * @param scansize	tama�o de imagen recorrida por cada paso
	 * @param flags	banda que se va a sustituir (Ctes de GeoRasterFile)
	 */
	protected void setRGBLine(BufferedImage image, int startX, int startY, int w, int h, int[] rgbArray,
			 int offset, int scansize, int flags) {
		int [] line = new int[rgbArray.length];
		image.getRGB(startX, startY, w, h, line, offset, scansize);
		if (flags == GeoRasterFile.RED_BAND)
			for (int i=0; i<line.length; i++)
				line[i] = (line[i] & 0x0000ffff) | (rgbArray[i] & 0xffff0000);
		else if (flags == GeoRasterFile.GREEN_BAND)
			for (int i=0; i<line.length; i++)
				line[i] = (line[i] & 0x00ff00ff) | (rgbArray[i] & 0xff00ff00);
		else if (flags == GeoRasterFile.BLUE_BAND)
			for (int i=0; i<line.length; i++)
				line[i] = (line[i] & 0x00ffff00) | (rgbArray[i] & 0xff0000ff);
		image.setRGB(startX, startY, w, h, line, offset, scansize);
	}

	/**
	 * Asigna al objeto Image la mezcla entre los valores que ya tiene y los valores
	 * con los dato de la imagen contenidos en el vector de enteros. De los valores RGB
	 * que ya contiene se mantienen las bandas que no coinciden con el valor de flags. La
	 * banda correspondiente a flags es sustituida por los datos del vector.
	 * @param image	imagen con los datos actuales
	 * @param startX	inicio de la posici�n en X dentro de la imagen
	 * @param startY	inicio de la posici�n en X dentro de la imagen
	 * @param w	Ancho de la imagen
	 * @param h	Alto de la imagen
	 * @param rgbArray	vector que contiene la banda que se va a sustituir
	 * @param offset	desplazamiento
	 * @param scansize	tama�o de imagen recorrida por cada paso
	 * @param origBand	Banda origen del GeoRasterFile
	 * @param destBandFlag	banda que se va a sustituir (Ctes de GeoRasterFile)
	 */
	protected void setRGBLine(BufferedImage image, int startX, int startY, int w, int h, int[] rgbArray,
			 int offset, int scansize, int origBand, int destBandFlag) {
		int [] line = new int[rgbArray.length];
		image.getRGB(startX, startY, w, h, line, offset, scansize);
		if (origBand == 0 && destBandFlag == GeoRasterFile.RED_BAND)
			for (int i=0; i<line.length; i++)
				line[i] = (line[i] & 0x0000ffff) | (rgbArray[i] & 0xffff0000);
		else if (origBand == 1 && destBandFlag == GeoRasterFile.GREEN_BAND)
			for (int i=0; i<line.length; i++)
				line[i] = (line[i] & 0x00ff00ff) | (rgbArray[i] & 0xff00ff00);
		else if (origBand == 2 && destBandFlag == GeoRasterFile.BLUE_BAND)
			for (int i=0; i<line.length; i++)
				line[i] = (line[i] & 0x00ffff00) | (rgbArray[i] & 0xff0000ff);

		else if (origBand == 0 && destBandFlag == GeoRasterFile.GREEN_BAND)
			for (int i=0; i<line.length; i++)
				line[i] = (line[i] & 0xffff00ff) | ((rgbArray[i] & 0x00ff0000) >> 8) ;
		else if (origBand == 0 && destBandFlag == GeoRasterFile.BLUE_BAND)
			for (int i=0; i<line.length; i++)
				line[i] = (line[i] & 0xffffff00) | ((rgbArray[i] & 0x00ff0000) >> 16);
		else if (origBand == 1 && destBandFlag == GeoRasterFile.RED_BAND)
			for (int i=0; i<line.length; i++)
				line[i] = (line[i] & 0xff00ffff) | ((rgbArray[i] & 0x0000ff00) << 8);

		else if (origBand == 1 && destBandFlag == GeoRasterFile.BLUE_BAND)
			for (int i=0; i<line.length; i++)
				line[i] = (line[i] & 0xffffff00) | ((rgbArray[i] & 0x0000ff00) >> 8);
		else if (origBand == 2 && destBandFlag == GeoRasterFile.RED_BAND)
			for (int i=0; i<line.length; i++)
				line[i] = (line[i] & 0xff00ffff) | ((rgbArray[i] & 0x000000ff) << 16);
		else if (origBand == 2 && destBandFlag == GeoRasterFile.GREEN_BAND)
			for (int i=0; i<line.length; i++)
				line[i] = (line[i] & 0xffff00ff) | ((rgbArray[i] & 0x000000ff) << 8);
		image.setRGB(startX, startY, w, h, line, offset, scansize);
	}

	private void showOnOpen() {
  		// Report en la apertura (quitar)
  		System.out.println("Fichero GDAL '"+getName()+"' abierto.");
  		System.out.println("Version = "+file.version);
  		System.out.println("   Size = ("+file.width+","+file.height+")");
  		try {
			System.out.println("   NumBands = ("+file.getRasterCount()+")");
		} catch (GdalException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
  		//file.pintaInfo();
  		file.pintaPaleta();

	}

	/* (non-Javadoc)
	 * @see org.cresques.io.GeoRasterFile#updateImage(int, int, org.cresques.cts.ICoordTrans, java.awt.Image, int, int)
	 */
	public Image updateImage(int width, int height, ICoordTrans rp, Image img, int origBand, int destBandFlag)throws SupersamplingNotSupportedException{
		int line, pRGBArray[] = null;

		if(mustVerifySize()){
			// Work out the correct aspect for the setView call.
			double dFileAspect = (double)v.width()/(double)v.height();
			double dWindowAspect = (double)width /(double)height;

			if (dFileAspect > dWindowAspect) {
			  height =(int)((double)width/dFileAspect);
			} else {
			  width = (int)((double)height*dFileAspect);
			}
		}

//		 Set the view
		boolean[] orientation = getOrientation();
		file.setView(v.minX(), v.maxY(), v.maxX(), v.minY(),
			width, height, orientation);
		setStep(file.stepArrayX, file.stepArrayY);

		if(width<=0)width=1;
		if(height<=0)height=1;

		pRGBArray = new int[width];
		try {
			setBand(RED_BAND,   rBandNr);
			setBand(GREEN_BAND, gBandNr);
			setBand(BLUE_BAND,  bBandNr);
			file.setAlpha(getAlpha());
			if(img!=null){
				if(orientation[1]){
					for (line=0; line < height; line++) {
						file.readLineRGBA(pRGBArray);
						setRGBLine((BufferedImage) img, 0, height - 1 - line, width, 1, pRGBArray, 0, width, origBand, destBandFlag);
					}
				}else{
					for (line=0; line < height; line++) {
						file.readLineRGBA(pRGBArray);
						setRGBLine((BufferedImage) img, 0, line, width, 1, pRGBArray, 0, width, origBand, destBandFlag);
					}
				}
				return img;
			}else{
				Image image = new BufferedImage(width, height, BufferedImage.TYPE_INT_ARGB);
				if(orientation[1]){
					for (line=0; line < height; line++) {
						file.readLineRGBA(pRGBArray);
						setRGBLine((BufferedImage) image, 0, height - 1 - line, width, 1, pRGBArray, 0, width);
					}
				}else{
					for (line=0; line < height; line++) {
						file.readLineRGBA(pRGBArray);
						setRGBLine((BufferedImage) image, 0, line, width, 1, pRGBArray, 0, width);
					}
				}
				return image;
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		return img;
	}

	/* (non-Javadoc)
	 * @see org.cresques.io.GeoRasterFile#getData(int, int, int)
	 */
	public Object getData(int x, int y, int band) {
		if(file != null){
			Object[] data = file.getData(x, y);
			return data[band];
		}
		return null;
	}

	/**
	 * Devuelve los datos de una ventana solicitada
	 * @param ulX	coordenada X superior izda.
	 * @param ulY	coordenada Y superior derecha.
	 * @param sizeX	tama�o en X de la ventana.
	 * @param sizeY tama�o en Y de la ventana.
	 * @param band	Banda solicitada.
	 */
	public byte[] getWindow(int ulX, int ulY, int sizeX, int sizeY, int band){

		return null;
	}

	public RasterBuf getWindowRaster(double x, double y, double w, double h, BandList bandList, RasterBuf rasterBuf) {
		Extent selectedExtent = new Extent(x, y, x + w, y - h);
		setView(selectedExtent);

		int width = 0;
		int height = 0;
		if(file.trans != null){
			width = (int)Math.abs(selectedExtent.width() / file.trans.adfgeotransform[1]);//(int)(selectedExtent.width() * file.width) / extent.width();
			height = (int)Math.abs(selectedExtent.height() / file.trans.adfgeotransform[5]);
		}else{
			width = (int)Math.abs(selectedExtent.width());
			height = (int)Math.abs(selectedExtent.height());
		}

		try {
			file.readWindow(rasterBuf, bandList, x, y, width, height);
		} catch (Exception e) {
			e.printStackTrace();
		}

		return rasterBuf;
	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.fmap.driver.GeoRasterFile#getWindowRaster(double, double, double, double, int, int, org.gvsig.fmap.driver.BandList, org.gvsig.fmap.driver.IBuffer)
	 */
	public RasterBuf getWindowRaster(double minX, double minY, double maxX, double maxY, int bufWidth, int bufHeight, BandList bandList, RasterBuf rasterBuf) {
		Extent selectedExtent = new Extent(minX, minY, maxX, maxY);
		setView(selectedExtent);

		double width = 0;
		double height = 0;
		if(getTransform() != null){
			width = (double)(Math.abs(selectedExtent.width() / getTransform()[1]));//(int)(selectedExtent.width() * file.width) / extent.width();
			height = (double)(Math.abs(selectedExtent.height() / getTransform()[5]));
		}else{
			width = (double)Math.abs(selectedExtent.width());
			height = (double)Math.abs(selectedExtent.height());
		}

		try {
			file.readWindow(rasterBuf, bandList, minX, maxY, maxX, minY, width, height, bufWidth, bufHeight);
		} catch (Exception e) {
			e.printStackTrace();
		}

		return rasterBuf;
	}

	public RasterBuf getWindowRaster(int x, int y, int w, int h, BandList bandList, RasterBuf rasterBuf) {
		try {
			setView(
			new Extent( Utilities.getMapRectFromPxRect(getExtent().toRectangle2D(),
						getWidth(),
						getHeight(),
						new Rectangle2D.Double(x, y, w, h)))
			);
			file.readWindow(rasterBuf, bandList, x, y, w, h);
		} catch (Exception e) {
			e.printStackTrace();
		}
		return rasterBuf;
	}

	public RasterBuf getWindowRasterWithNoData(double x, double y, double w, double h, BandList bandList, RasterBuf rasterBuf) {
		Extent selectedExtent = new Extent(x, y, x + w, y - h);
		setView(selectedExtent);

		try {
			file.readWindowWithNoData(rasterBuf, bandList, x, y, x + w, y - h, rasterBuf.getWidth(), rasterBuf.getHeight());
		} catch (Exception e) {
			e.printStackTrace();
		}

		return rasterBuf;
	}

	/**
	 * Obtiene la zona (Norte / Sur)
	 * @return true si la zona es norte y false si es sur
	 */

	public boolean getZone(){

		return false;
	}

	/**
	 *Devuelve el n�mero de zona UTM
	 *@return N�mero de zona
	 */

	public int getUTM(){

		return 0;
	}

	/**
	 * Obtiene el sistema de coordenadas geograficas
	 * @return Sistema de coordenadas geogr�ficas
	 */
	public String getGeogCS(){
		return new String("");
	}

	/**
	 * Devuelve el tama�o de bloque
	 * @return Tama�o de bloque
	 */
	public int getBlockSize(){
		if(file != null)
			return file.getBlockSize();
		else
			return 0;
	}

	/**
	 * Obtiene el objeto que contiene los metadatos
	 */
	public Metadata getMetadata() {
		if(file != null)
			return file.getMetadataJavaObject();
		else
			return null;
	}

	/**
	 * Obtiene el flag que dice si la imagen est� o no georreferenciada
	 * @return true si est� georreferenciada y false si no lo est�.
	 */
	public boolean isGeoreferenced() {
		if(file != null)
			return file.isGeoreferenced();
		else
			return false;
	}

	/**
	 * Informa de si el driver ha supersampleado en el �ltimo dibujado. Es el driver el que colocar�
	 * el valor de esta variable cada vez que dibuja.
	 * @return true si se ha supersampleado y false si no se ha hecho.
	 */
	public boolean isSupersampling() {
		if(file != null)
			return file.isSupersampling;
		else
			return false;
	}

	/**
	 * Obtiene los par�metros de la transformaci�n af�n que corresponde con los elementos de
	 * un fichero tfw.
	 * <UL>
	 * <LI>[1]tama�o de pixel en X</LI>
	 * <LI>[2]rotaci�n en X</LI>
	 * <LI>[4]rotaci�n en Y</LI>
	 * <LI>[5]tama�o de pixel en Y</LI>
	 * <LI>[0]origen en X</LI>
	 * <LI>[3]origen en Y</LI>
	 * </UL>
	 * Este m�todo debe ser reimplementado por el driver si tiene esta informaci�n. En principio
	 * Gdal es capaz de proporcionarla de esta forma.
	 *
	 * En caso de que exista fichero .rmf asociado al raster pasaremos de la informaci�n de georreferenciaci�n
	 * del .tfw y devolveremos la que est� asociada al rmf
	 * @return vector de double con los elementos de la transformaci�n af�n.
	 */
	public double[] getTransform(){
		if(file != null && file.trans != null && !this.rmfExists())
			return file.trans.adfgeotransform;
		else{
			if(this.rmfExists){
				double[] rmfGeoref = {	rmfTransform.getTranslateX(),
										rmfTransform.getScaleX(),
										rmfTransform.getShearX(),
										rmfTransform.getTranslateY(),
										rmfTransform.getShearY(),
										rmfTransform.getScaleY()};
				return rmfGeoref;
			}
			return null;
		}

	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.fmap.driver.GeoRasterFile#rasterToWorld(java.awt.geom.Point2D)
	 */
	public Point2D rasterToWorld(Point2D pt) {
		return file.rasterToWorld(pt);
	}

	/*
	 *  (non-Javadoc)
	 * @see org.gvsig.fmap.driver.GeoRasterFile#worldToRaster(java.awt.geom.Point2D)
	 */
	public Point2D worldToRaster(Point2D pt){
		return file.worldToRaster(pt);
	}

	public void readPalette(){
		file.readPalette();
	}

}



