/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io;

import java.awt.geom.Point2D;
import java.io.IOException;
import java.util.Vector;

import org.cresques.io.data.BandList;
import org.cresques.io.data.RasterBuf;
import org.cresques.io.datastruct.Metadata;
import org.cresques.io.datastruct.Palette;
import org.cresques.px.Extent;
import org.cresques.util.Utilities;

import es.gva.cit.jgdal.Gdal;
import es.gva.cit.jgdal.GdalBuffer;
import es.gva.cit.jgdal.GdalException;
import es.gva.cit.jgdal.GdalRasterBand;
import es.gva.cit.jgdal.GeoTransform;

/**
 * Soporte 'nativo' para ficheros desde GDAL.
 * Este conjunto de funcionalidades est tomado de manera casi literal
 * del soporte para ECW de ermapper.<br>
 * Probablemente esto debera formar parte del JNI que recubre a la
 * librera en C extraida de gdal.<br>
 * Lo pongo aqu a manera de ejemplo de como atacar un formato binario
 * desde Java.<br><br>
 * @author Luis W. Sevilla.
 */

class GdalNative extends Gdal {
	static boolean 				WITH_OVERVIEWS = true;
	protected GdalFile			driver = null;
	protected String 				ext = "";
	protected String 				fileName = null;
	/**
	 * Nombre corto del driver de gdal
	 */
	protected String 				shortName = "";
	public 	GeoTransform 		trans = null;
	/**
	 * Contorno en coordenadas geogrficas. (y Extent del raster).
	 */
	public Contour 				bBoxRot = new Contour();
	/**
	 * Contorno en coordenadas geogrficas sin rotacin aplicada. Esto es util para poder
	 * calcular los pixeles necesarios que se van a leer del raster. Cuando el raster no tiene
	 * rotacin coincide con esq.
	 */
	public Contour				bBoxWithoutRot = new Contour();
	public int 					width = 0, height = 0;
	public double 				originX = 0D, originY = 0D;
	public String 				version = "";
	private int 				alpha = 0;
	protected int 				rBandNr = 1, gBandNr = 2, bBandNr = 3, aBandNr = 4;
	private int 				dataType = GDT_Byte;
	/**
	 * Metadatos leidos de la imagen
	 */
	protected Metadata			metadata = null;
	protected boolean 			georeferenced = true;

	/**
	 * Vectores que contiene los desplazamientos de un pixel cuando hay supersampling.
	 * , es decir el nmero de pixels de pantalla que tiene un pixel de imagen. Como todos
	 * los pixeles no tienen el mismo ancho y alto ha de meterse en un array y no puede ser
	 * una variable. Adems hay que tener en cuenta que el primer y ltimo pixel son de
	 * distinto tamao que el resto.
	 */
	public int[]				stepArrayX = null, stepArrayY = null;
	protected GdalRasterBand[] 	gdalBands = null;
	private double				lastReadLine = -1;
	protected int 				currentFullWidth = -1;
	protected int 				currentFullHeight = -1;
	private int 				currentViewWidth = -1;
	private int 				currentViewHeight = -1;
	private double 				currentViewX = 0D;
	private double 				currentViewY = 0D;
	private double 				viewportScaleX = 0D;
	private double 				viewportScaleY = 0D;
	private double 				wcWidth = 0D;
	private double 				stepX = 0D;
	private double 				stepY = 0D;
	public boolean  			isSupersampling = false;
	private boolean[] 			orientation;

	/**
	 * Overview usada en el ltimo setView
	 */
	int currentOverview = -1;

	// Polilinea con extent
	public class Contour extends Vector {
		final private static long serialVersionUID = -3370601314380922368L;
		public double minX = Double.MAX_VALUE, minY = Double.MAX_VALUE;
		public double maxX = -Double.MAX_VALUE, maxY = -Double.MAX_VALUE;
		public Contour() {
			super();
		}
		public void add(Point2D pt) {
			super.add(pt);
			if (pt.getX() > maxX) maxX = pt.getX();
			if (pt.getX() < minX) minX = pt.getX();
			if (pt.getY() > maxY) maxY = pt.getY();
			if (pt.getY() < minY) minY = pt.getY();
		}
	}

	public GdalNative(){
		super();
	}

	public GdalNative(String fName, GdalFile driver) throws GdalException, IOException {
		super();
		this.driver = driver;
		init(fName);
	}

	public Point2D rasterToWorld(Point2D pt) {
		double x = bBoxWithoutRot.minX + ((pt.getX() * (bBoxWithoutRot.maxX - bBoxWithoutRot.minX)) / width);
		double y = bBoxWithoutRot.maxY - ((pt.getY() * (bBoxWithoutRot.maxY - bBoxWithoutRot.minY)) / height);
		Point2D ptRes = new Point2D.Double(x, y);
		return ptRes;
	}

	/**
	 * <P>
	 * Calcula la bounding box en la que est metido el raster teniendo en cuenta
	 * el tamao de pixel y la rotacin. Esto lo hace con los valores de transformacin
	 * leidos por gdal en el vector de 6 elementos adfGeoTransform donde cada elemento
	 * del vector represnta los siguientes valores
	 * </P>
	 * <UL>
	 * <LI>0-origen X</LI>
	 * <LI>1-tamao de pixel X</LI>
	 * <LI>2-shear en X</LI>
	 * <LI>3-origen Y</LI>
	 * <LI>4-shear en Y</LI>
	 * <LI>5-Tamao de pixel Y</LI>
	 * </UL>
	 * <P>
	 * Para el calculo de una esquina aplicamos la formula siguiente:<BR>
	 * PtoX = originX + pixelSizeX * x + shearX * y;<BR>
	 * PtoY = originY + shearY * x + pixelSizeY * y;<BR>
	 * Aplicandolo a las cuatro esquinas sustituimos en cada una de ellas por.
	 * </P>
	 * <UL>
	 * <LI>Esquina superior izquierda: x = 0; y = 0;</LI>
	 * <LI>Esquina superior derecha: x = MaxX; y = 0;</LI>
	 * <LI>Esquina inferior izquierda: x = 0; y = MaxY;</LI>
	 * <LI>Esquina inferior derecha: x = MaxX; y = MaxY;</LI>
	 * </UL>
	 * <P>
	 * quedandonos en los cuatro casos:
	 * </P>
	 * <UL>
	 * <LI>Esquina superior izquierda: originX; originY;</LI>
	 * <LI>Esquina superior derecha: PtoX = originX + pixelSizeX * x; PtoY = originY + shearY * x;</LI>
	 * <LI>Esquina inferior izquierda:  PtoX = originX + shearX * y; PtoY = originY + pixelSizeY * y;</LI>
	 * <LI>Esquina inferior derecha: PtoX = originX + pixelSizeX * x + shearX * y; PtoY = originY + shearY * x + pixelSizeY * y;</LI>
	 * </UL>
	 *
	 */
	protected void boundingBoxFromGeoTransform(){
		double geoX = 0D, geoY = 0D;

		//Upper left corner
		bBoxRot.add(new Point2D.Double(trans.adfgeotransform[0], trans.adfgeotransform[3]));

		//Lower left corner
		geoX = trans.adfgeotransform[0] +  trans.adfgeotransform[2] * height;
		geoY = trans.adfgeotransform[3] +  trans.adfgeotransform[5] * height;
		bBoxRot.add(new Point2D.Double(geoX, geoY));

		//Upper right corner
		geoX = trans.adfgeotransform[0] + trans.adfgeotransform[1] * width;
		geoY = trans.adfgeotransform[3] + trans.adfgeotransform[4] * width;
		bBoxRot.add(new Point2D.Double(geoX, geoY));

		//Lower right corner
		geoX = trans.adfgeotransform[0] + trans.adfgeotransform[1] * width + trans.adfgeotransform[2] * height;
		geoY = trans.adfgeotransform[3] + trans.adfgeotransform[4] * width + trans.adfgeotransform[5] * height;
		bBoxRot.add(new Point2D.Double(geoX, geoY));

		//TODO: OJO! con coordenadas geogrficas
	}

	/**
	 * Calcula la bounding box en la que est metido el raster teniendo en cuenta
	 * el tamao de pixel y la rotacin.
	 */
	protected void boundingBoxWithoutRotation(){
		double ox = trans.adfgeotransform[0];
		double oy = trans.adfgeotransform[3];
		double resx = trans.adfgeotransform[1];
		double resy = trans.adfgeotransform[5];

		bBoxWithoutRot.add(new Point2D.Double(ox, oy));
		bBoxWithoutRot.add(new Point2D.Double(ox + resx * width, oy));
		bBoxWithoutRot.add(new Point2D.Double(ox, oy + resy * height));
		bBoxWithoutRot.add(new Point2D.Double(ox + resx * width, oy + resy * height));

		//TODO: OJO! con coordenadas geogrficas
	}

	private void init(String fName) throws GdalException, IOException {
		fileName = fName;
		open(fName,GA_ReadOnly);
		if(getPtro() == -1)
			throw new GdalException("Formato de fichero no valido");
		ext = fName.toLowerCase().substring(fName.lastIndexOf('.')+1);
		if (ext.compareTo("tif") == 0)
			WITH_OVERVIEWS = false;
		currentFullWidth = width = getRasterXSize();
		currentFullHeight = height = getRasterYSize();
		setDataType(this.getRasterBand(1).getRasterDataType());
		shortName = getDriverShortName();
		metadata = new Metadata(getMetadata());

		//Asignamos la interpretacin de color leida por gdal a cada banda. Esto nos sirve
		//para saber que banda de la imagen va asignada a cada banda de visualizacin (ARGB)
		metadata.initColorInterpretation(getRasterCount());
		metadata.initNoDataByBand(getRasterCount());
	    for(int i = 0; i < getRasterCount(); i++){
	    	GdalRasterBand rb = getRasterBand(i + 1);
	    	String colorInt = getColorInterpretationName(rb.getRasterColorInterpretation());
	    	metadata.setNoDataValue(i, rb.getRasterNoDataValue());
	    	metadata.setColorInterpValue(i, colorInt);
	    	if(colorInt.equals("Red"))
	    		rBandNr = i + 1;
	    	if(colorInt.equals("Green"))
	    		gBandNr = i + 1;
	    	if(colorInt.equals("Blue"))
	    		bBandNr = i + 1;
	    	if(colorInt.equals("Alpha"))
	    		aBandNr = i + 1;
	    }

		try{
			trans = getGeoTransform();

			boolean isCorrect = false;
			for(int i = 0; i < trans.adfgeotransform.length; i++)
				if(trans.adfgeotransform[i] != 0)
					isCorrect = true;
			if(!isCorrect)
				throw new GdalException("");

			boundingBoxWithoutRotation();
			boundingBoxFromGeoTransform();

			this.georeferenced = true;
		}catch(GdalException exc){
			bBoxRot.add(new Point2D.Double(0, 0));
			bBoxRot.add(new Point2D.Double(width, 0));
			bBoxRot.add(new Point2D.Double(0, height));
			bBoxRot.add(new Point2D.Double(width, height));
			bBoxWithoutRot = bBoxRot;
			this.georeferenced = false;
		}
		readPalette();
	}

	public void readPalette(){
		//Cargamos la tabla de color si la tiene
		try{
			gdalBands = new GdalRasterBand[1];
			gdalBands[0] = getRasterBand(1);
	        if(gdalBands[0] != null && gdalBands[0].getRasterColorTable() != null){
	        	Palette palette = new Palette();
	        	palette.createPaletteFromGdalColorTable(gdalBands[0].getRasterColorTable());
	        	driver.setPalette(palette);
	        }
		}catch(GdalException exc){
			//No cargamos la tabla de color
		}
	}

	/**
	 * Asigna el valor de Alpha
	 * @param a
	 */
	public void setAlpha(int a) {
		alpha = a;
	}

	/**
	 * Asigna el tipo de dato
	 * @param dt entero que representa el tipo de dato
	 */
	public void setDataType(int dt) {
		dataType = dt;
	}

	/**
	 * Obtiene el tipo de dato
	 * @return entero que representa el tipo de dato
	 */
	public int getDataType() {
		return dataType;
	}

	// Supone rasters no girados
	public Point2D worldToRaster(Point2D pt) {
		double x = (((double) currentFullWidth) / (bBoxWithoutRot.maxX - bBoxWithoutRot.minX)) * (pt.getX() - bBoxWithoutRot.minX);
		double y = (((double) currentFullHeight) / (bBoxWithoutRot.maxY - bBoxWithoutRot.minY)) * (bBoxWithoutRot.maxY - pt.getY());
		Point2D ptRes = new Point2D.Double(x, y);
		return ptRes;
	}

	/**
	 * Calcula el overview a usar. Hay que tener en cuenta que tenemos que tener calculadas las variables
	 * viewPortScale, currentFullWidth y currentFulHeight
	 * @param coordenada pixel expresada en double que indica la posicin superior izquierda
	 * @throws GdalException
	 */
	private void calcOverview(Point2D tl, Point2D br, boolean[] orientation) throws GdalException{
		gdalBands[0] = getRasterBand(1);
		currentOverview = -1;
		if (WITH_OVERVIEWS && gdalBands[0].getOverviewCount() > 0) {
			GdalRasterBand ovb = null;
			for (int i = gdalBands[0].getOverviewCount()-1; i > 0; i--) {
				ovb = gdalBands[0].getOverview(i);
				if (ovb.getRasterBandXSize()>getRasterXSize()*viewportScaleX) {
					currentOverview = i;
            		viewportScaleX *= ((double) width/(double) ovb.getRasterBandXSize());
            		viewportScaleY *= ((double) height/(double) ovb.getRasterBandYSize());
            		stepX = 1D/viewportScaleX;
            		stepY = 1D/viewportScaleY;
            		currentFullWidth = ovb.getRasterBandXSize();
            		currentFullHeight = ovb.getRasterBandYSize();
            		if(!orientation[0])//Invierte la orientacin en X
            			currentViewX = (width - tl.getX()) - (br.getX()-tl.getX());
            		else
            			currentViewX = tl.getX();
            		if(orientation[1])//Invierte la orientacin en Y
            			lastReadLine = (height - tl.getY()) - (br.getY()-tl.getY());
            		else
            			lastReadLine = tl.getY();
            		break;
				}
			}
		}
	}

	public void setView(double dWorldTLX, double dWorldTLY,
            double dWorldBRX, double dWorldBRY,
            int nWidth, int nHeight, boolean[] orientation) {
		this.orientation = orientation;
		currentFullWidth = width;
		currentFullHeight = height;
		Point2D tl = worldToRaster(new Point2D.Double(dWorldTLX, dWorldTLY));
		Point2D br = worldToRaster(new Point2D.Double(dWorldBRX, dWorldBRY));
		// Calcula cual es la primera lnea a leer;
		currentViewWidth = nWidth;
		currentViewHeight = nHeight;
		wcWidth = Math.abs(br.getX() - tl.getX());

		if(!orientation[0]) //Invierte la orientacin en X
			currentViewX = (width - tl.getX()) - (br.getX()-tl.getX());
		else
			currentViewX = tl.getX();

		viewportScaleX = (double) currentViewWidth/(br.getX()-tl.getX());
		viewportScaleY = (double) currentViewHeight/(br.getY()-tl.getY());
		stepX = 1D/viewportScaleX;
		stepY = 1D/viewportScaleY;

		if(orientation[1])//Invierte la orientacin en Y
			lastReadLine = (height - tl.getY()) - (br.getY()-tl.getY());
		else
			lastReadLine = tl.getY();

		//Para lectura del renderizado (ARGB). readWindow selecciona las bandas que necesita.
		try {
			// calcula el overview a usar
			gdalBands = new GdalRasterBand[4];
			calcOverview(tl, br, orientation);
			calcArraySteps();

			// Selecciona las bandas y los overviews necesarios
			gdalBands[0] = getRasterBand(rBandNr);
			gdalBands[1] = gdalBands[0];
			gdalBands[2] = gdalBands[1];
			setDataType(gdalBands[0].getRasterDataType());
			if(this.getRasterCount() >= 2) {
				gdalBands[1] = getRasterBand(gBandNr);
				gdalBands[2] = gdalBands[1];
			}
			if(this.getRasterCount() >= 3)
				gdalBands[2] = getRasterBand(bBandNr);
			if(metadata.isAlphaBand())
				gdalBands[3] = getRasterBand(aBandNr);


			if (currentOverview > 0) {
				gdalBands[0] = gdalBands[0].getOverview(currentOverview);
				gdalBands[1] = gdalBands[0];
				gdalBands[2] = gdalBands[1];
				if(this.getRasterCount() >= 2) {
					gdalBands[1] = gdalBands[1].getOverview(currentOverview);
					gdalBands[2] = gdalBands[1];
				}
				if(this.getRasterCount() >= 3)
					gdalBands[2] = gdalBands[2].getOverview(currentOverview);
				if(metadata.isAlphaBand())
					gdalBands[3] = gdalBands[3].getOverview(currentOverview);

			}

		} catch (GdalException e) {
			e.printStackTrace();
		}
	}

	/**
	 * Esta funcin calcula los arrays de steps en X e Y para que cuando hay supersampleo
	 * se aplique el filtro solo a la esquina superior izquierda de cada pixel.
	 */
	private void calcArraySteps(){
		if(stepX < 1 && stepY < 1){
			isSupersampling = true;
			int w = (int) (Math.ceil(((double)currentViewWidth) * stepX) + 1);
			this.stepArrayX = new int[w];
			for (double j =  Math.abs(currentViewX - ((int)currentViewX)); j < w; j += stepX)
					stepArrayX[(int)(j)] ++;

			int h = (int) (Math.ceil(((double)currentViewHeight) * stepY) + 1);
			this.stepArrayY = new int[h];
			for (double j =  Math.abs(lastReadLine - ((int)lastReadLine)); j < h; j += stepY)
				stepArrayY[(int)(j)] ++;
		}else{
			isSupersampling = false;
			this.stepArrayX = this.stepArrayY = null;
		}
	}

	int lastY = -1;

	/**
	 * Lee una lnea de bytes
	 * @param line Buffer donde se cargan los datos
	 * @param initOffset Desplazamiento inicial desde el margen inzquierdo. Esto es necesario para cuando
	 * se supersamplea ya que cada pixel de imagen ocupa muchos pixeles de pantalla y puede empezar a dibujarse
	 * por la izquierda a mitad de pixel
	 * @param gdalBuffer Buffer con la lnea de datos original
	 */
	private void readLine(byte[][] line, double initOffset, GdalBuffer[] gdalBuffer){
		double j = 0D;
  		int i = 0;
		for (int iBand = 0; iBand < gdalBuffer.length; iBand++){
			for (i = 0, j = initOffset; i < currentViewWidth && j < gdalBuffer[0].getSize(); i++, j+=stepX) {
				line[iBand][i] = gdalBuffer[iBand].buffByte[(int) j];
			}
		}
	}

	/**
	 * Lee una lnea de shorts
	 * @param line Buffer donde se cargan los datos
	 * @param initOffset Desplazamiento inicial desde el margen inzquierdo. Esto es necesario para cuando
	 * se supersamplea ya que cada pixel de imagen ocupa muchos pixeles de pantalla y puede empezar a dibujarse
	 * por la izquierda a mitad de pixel
	 * @param gdalBuffer Buffer con la lnea de datos original
	 */
	private void readLine(short[][] line, double initOffset, GdalBuffer[] gdalBuffer){
  		double j = 0D;
  		int i = 0;
		for (int iBand = 0; iBand < gdalBuffer.length; iBand++){
			for (i = 0, j = initOffset; i < currentViewWidth && j < gdalBuffer[0].getSize(); i++, j+=stepX) {
				line[iBand][i] = (short)(gdalBuffer[iBand].buffShort[(int) j] & 0xffff);
			}
		}
	}

	/**
	 * Lee una lnea de ints
	 * @param line Buffer donde se cargan los datos
	 * @param initOffset Desplazamiento inicial desde el margen inzquierdo. Esto es necesario para cuando
	 * se supersamplea ya que cada pixel de imagen ocupa muchos pixeles de pantalla y puede empezar a dibujarse
	 * por la izquierda a mitad de pixel
	 * @param gdalBuffer Buffer con la lnea de datos original
	 */
	private void readLine(int[][] line, double initOffset, GdalBuffer[] gdalBuffer){
		double j = 0D;
  		int i = 0;
		for (int iBand = 0; iBand < gdalBuffer.length; iBand++){
			for (i = 0, j = initOffset; i < currentViewWidth && j < gdalBuffer[0].getSize(); i++, j+=stepX) {
				line[iBand][i] = (gdalBuffer[iBand].buffInt[(int) j] & 0xffffffff);
			}
		}
	}

	/**
	 * Lee una lnea de float
	 * @param line Buffer donde se cargan los datos
	 * @param initOffset Desplazamiento inicial desde el margen izquierdo. Esto es necesario para cuando
	 * se supersamplea ya que cada pixel de imagen ocupa muchos pixeles de pantalla y puede empezar a dibujarse
	 * por la izquierda a mitad de pixel
	 * @param gdalBuffer Buffer con la lnea de datos original
	 */
	private void readLine(float[][] line, double initOffset, GdalBuffer[] gdalBuffer){
		double j = 0D;
  		int i = 0;
		for (int iBand = 0; iBand < gdalBuffer.length; iBand++){
			for (i = 0, j = initOffset; i < currentViewWidth && j < gdalBuffer[0].getSize(); i++, j+=stepX) {
				line[iBand][i] = gdalBuffer[iBand].buffFloat[(int) j];
			}
		}
	}

	/**
	 * Lee una lnea de doubles
	 * @param line Buffer donde se cargan los datos
	 * @param initOffset Desplazamiento inicial desde el margen inzquierdo. Esto es necesario para cuando
	 * se supersamplea ya que cada pixel de imagen ocupa muchos pixeles de pantalla y puede empezar a dibujarse
	 * por la izquierda a mitad de pixel
	 * @param gdalBuffer Buffer con la lnea de datos original
	 */
	private void readLine(double[][] line, double initOffset, GdalBuffer[] gdalBuffer){
		double j = 0D;
  		int i = 0;
		for (int iBand = 0; iBand < gdalBuffer.length; iBand++){
			for (i = 0, j = initOffset; i < currentViewWidth && j < gdalBuffer[0].getSize(); i++, j+=stepX) {
				line[iBand][i] = gdalBuffer[iBand].buffDouble[(int) j];
			}
		}
	}

	public void readLine(Object line) throws GdalException {
        int w = (int) (Math.ceil(((double)currentViewWidth)*stepX) + 1);
        int x = (int) (currentViewX);
        int y = (int) (lastReadLine);
        GdalBuffer r = null, g = null, b = null;
        GdalBuffer a = new GdalBuffer();

        while(y >= gdalBands[0].getRasterBandYSize())
        	y--;

        if (x+w > gdalBands[0].getRasterBandXSize())
        	w = gdalBands[0].getRasterBandXSize()-x;

        if(gdalBands[0].getRasterColorTable() != null){
        	/*Palette palette = new Palette();
        	palette.createPaletteFromGdalColorTable(gdalBands[0].getRasterColorTable());
        	driver.setPalette(palette);*/
        	r = gdalBands[0].readRaster(x, y, w, 1, w, 1, dataType);
        }else{
        	a.buffByte = new byte[w];
			r = gdalBands[0].readRaster(x, y, w, 1, w, 1, dataType);
			g = b = r;
			if (getRasterCount() > 1 && gdalBands[1] != null)
	    		g = gdalBands[1].readRaster(x, y, w, 1, w, 1, dataType);
			if (getRasterCount() > 2 && gdalBands[2] != null)
	    		b = gdalBands[2].readRaster(x, y, w, 1, w, 1, dataType);
        }

        lastReadLine += stepY;

  		double initOffset =  Math.abs(currentViewX - ((int)currentViewX));
  		GdalBuffer[] bands = {r, g, b};

  		if (dataType == GDT_Byte)
  			readLine((byte[][])line, initOffset, bands);
		else if (dataType == GDT_CInt16 || dataType == GDT_Int16  || dataType == GDT_UInt16)
			readLine((short[][])line, initOffset, bands);
		else if (dataType == GDT_CInt32 || dataType == GDT_Int32  || dataType == GDT_UInt32)
			readLine((int[][])line, initOffset, bands);
  		else if(dataType == GDT_Float32 || dataType == GDT_CFloat32)
  			readLine((float[][])line, initOffset, bands);
  		else if(dataType == GDT_Float64 || dataType == GDT_CFloat64)
  			readLine((double[][])line, initOffset, bands);

		return;
	}

	/**
	 * Lee una lnea y la guarda cada elemento sobre un entero. Este entero representa
	 * un valor ARGB
	 * @param line Buffer sobre el que se escribe la linea
	 * @return
	 * @throws GdalException
	 */
	int readLineRGBA(int[] line) throws GdalException {
		int err = 0;

        int w = (int) (Math.ceil(((double)currentViewWidth)*stepX) + 1);
        int x = (int) currentViewX;
        int y = (int) lastReadLine;
        GdalBuffer r = null, g = null, b = null, p = null;
        GdalBuffer a = new GdalBuffer();

        while(y >= gdalBands[0].getRasterBandYSize())
        	y--;

        if (x+w > gdalBands[0].getRasterBandXSize())
        	w = gdalBands[0].getRasterBandXSize()-x;

        if(gdalBands[0].getRasterColorTable() != null){
        	/*Palette palette = new Palette();
        	palette.createPaletteFromGdalColorTable(gdalBands[0].getRasterColorTable());
        	driver.setPalette(palette);*/
        	r = gdalBands[0].readRaster(x, y, w, 1, w, 1, dataType);
        }else{

        	r = gdalBands[0].readRaster(x, y, w, 1, w, 1, dataType);
        	g = b = r;
			if (getRasterCount() > 1 && gdalBands[1] != null)
	    		g = gdalBands[1].readRaster(x, y, w, 1, w, 1, dataType);
			if (getRasterCount() > 2 && gdalBands[2] != null)
	    		b = gdalBands[2].readRaster(x, y, w, 1, w, 1, dataType);

        	if(metadata.isAlphaBand()){
        		a = gdalBands[3].readRaster(x, y, w, 1, w, 1, GDT_Byte);
        	}else{
    			a.buffByte = new byte[w];
    			for (int i = 0;i < w;i++)
    				a.buffByte[i] = (byte)255;
        	}
        }
  		lastReadLine += stepY;

  		int i=0;
  		double j =  Math.abs(currentViewX - ((int)currentViewX));
		int alpha = (this.alpha & 0xff) << 24;

		if(orientation[0]){ //Pixel size en X positivo
			if (dataType == GDT_Byte){
	  			if(gdalBands[0].getRasterColorTable() != null){
	  				for (i=0; i<currentViewWidth && j<r.getSize(); i++, j+=stepX) {
		      			int jInt = (int)(j);
		      			line[i] = (alpha) + ((r.buffByte[jInt] & 0xff) << 16) + ((r.buffByte[jInt] & 0xff) << 8) + (r.buffByte[jInt] & 0xff);
		      		}
	  			}else{
	  				for (i=0; i<currentViewWidth && j<r.getSize(); i++, j+=stepX) {
		      			int jInt = (int)(j);
		      			line[i] = (alpha & ((a.buffByte[jInt])& 0xff) << 24) + ((r.buffByte[jInt] & 0xff) << 16) + ((g.buffByte[jInt] & 0xff) << 8) + (b.buffByte[jInt] & 0xff);
		      		}
	  			}
	  		}else if (dataType == GDT_CInt16 || dataType == GDT_Int16  || dataType == GDT_UInt16){
	  			if (g == null) // Sibgle Band (Typical DEM)
	  		  		for (i=0; i<currentViewWidth && j<r.getSize(); i++, j+=stepX) {
	  		  			int jInt = (int)(j);
		      			line[i] = (alpha & ((a.buffByte[jInt])& 0xff) << 24) + r.buffShort[jInt];
		      		}
	  			else { // Multiband
	  				// System.err.println("Raster 16bits multibanda");
		      		for (i=0; i<currentViewWidth && j<r.getSize(); i++, j+=stepX) {
		      			int jInt = (int)(j);
		      			line[i] = (alpha & ((a.buffByte[jInt])& 0xff) << 24) | (((r.buffShort[jInt] & 0xfff0) << 12) & 0xff0000 ) |
										  						 (((g.buffShort[jInt] & 0xfff0) << 4 ) & 0xff00 ) |
										  						 (((b.buffShort[jInt] & 0xfff0) >> 4 ) & 0xff );
		      		}
	  			}
	  		}
		}else{ //Pixel size en X negativo
			if (dataType == GDT_Byte){
	  			if(gdalBands[0].getRasterColorTable() != null){
	  				for (i=currentViewWidth - 1; i>=0 && j<r.getSize(); i--, j+=stepX)  {
		      			int jInt = (int)(j);
		      			line[i] = (alpha) + ((r.buffByte[jInt] & 0xff) << 16) + ((r.buffByte[jInt] & 0xff) << 8) + (r.buffByte[jInt] & 0xff);
		      		}
	  			}else{
	  				for (i=currentViewWidth - 1; i>=0 && j<r.getSize(); i--, j+=stepX)  {
		      			int jInt = (int)(j);
		      			line[i] = (alpha & ((a.buffByte[jInt])& 0xff) << 24) + ((r.buffByte[jInt] & 0xff) << 16) + ((g.buffByte[jInt] & 0xff) << 8) + (b.buffByte[jInt] & 0xff);
		      		}
	  			}
	  		}else if (dataType == GDT_CInt16 || dataType == GDT_Int16  || dataType == GDT_UInt16){
	  			if (g == null) // Sibgle Band (Typical DEM)
	  		  		for (i=currentViewWidth - 1; i>=0 && j<r.getSize(); i--, j+=stepX) {
	  		  			int jInt = (int)(j);
		      			line[i] = (alpha & ((a.buffByte[jInt])& 0xff) << 24) + r.buffShort[jInt];
		      		}
	  			else { // Multiband
	  				// System.err.println("Raster 16bits multibanda");
		      		for (i=currentViewWidth - 1; i>=0 && j<r.getSize(); i--, j+=stepX) {
		      			int jInt = (int)(j);
		      			line[i] = (alpha & ((a.buffByte[jInt])& 0xff) << 24) | (((r.buffShort[jInt] & 0xfff0) << 12) & 0xff0000 ) |
										  						 (((g.buffShort[jInt] & 0xfff0) << 4 ) & 0xff00 ) |
										  						 (((b.buffShort[jInt] & 0xfff0) >> 4 ) & 0xff );
		      		}
	  			}
	  		}
		}
		return err;
	}

	/**
	 * Lee una ventana de datos sin resampleo a partir de coordenadas reales.
	 * @param buf Buffer donde se almacenan los datos
	 * @param bandList Lista de bandas que queremos leer y sobre que bandas del buffer de destino queremos escribirlas
	 * @param dWorldTLX Posicin X en pixeles
	 * @param dWorldTLY Posicin Y en pixeles
	 * @param w Ancho en pixeles
	 * @param h Alto en pixeles
	 * @throws GdalException
	 */
	public void readWindow(RasterBuf buf, BandList bandList, double dWorldTLX, double dWorldTLY,
            				int nWidth, int nHeight) throws GdalException {
		Point2D tl = worldToRaster(new Point2D.Double(dWorldTLX, dWorldTLY));

		gdalBands = new GdalRasterBand[getRasterCount()];
		isSupersampling = false;
		if(gdalBands.length == 0)
			return;

		try {
			// Selecciona las bandas
			gdalBands[0] = getRasterBand(1);
			setDataType(gdalBands[0].getRasterDataType());
			for(int iBand = 1; iBand < gdalBands.length; iBand++)
				gdalBands[iBand] = getRasterBand(iBand + 1);

		} catch (GdalException e) {
			e.printStackTrace();
		}

        int x = (int) Math.ceil(tl.getX());
        int y = (int) Math.ceil(tl.getY());

        if ((x + nWidth) > gdalBands[0].getRasterBandXSize())
        	nWidth = gdalBands[0].getRasterBandXSize() - x;

        if ((y + nHeight) > gdalBands[0].getRasterBandYSize())
        	nHeight = gdalBands[0].getRasterBandYSize() - y;

		int yMax = y + nHeight;
        readData(buf, bandList, x, y, nWidth, yMax);
	}

	/**
	 * Lee una ventana de datos con resampleo a partir de coordenadas reales. Este mtodo lee la
	 * ventana de una vez cargando los datos de un golpe en el buffer. Las coordenadas se solicitan
	 * en coordenadas del mundo real por lo que estas pueden caer en cualquier parte de un pixel.
	 * Esto se hace ms evidente cuando supersampleamos en la peticin, es decir el buffer de de
	 * mayor tamao que el nmero de pixels solicitado.
	 *
	 * Para resolver esto escribiremos con la funcin readRaster los datos sobre un buffer mayor
	 * que el solicitado. Despus calcularemos el desplazamiento en pixels dentro de este buffer
	 * de mayor tamao hasta llegar a la coordenada real donde comienza la peticin real que ha
	 * hecho el usuario. Esto es as porque cuando supersampleamos no queremos los pixeles del
	 * raster de disco completos sino que en los bordes del buffer quedan cortados.
	 *
	 * @param buf Buffer donde se almacenan los datos
	 * @param bandList Lista de bandas que queremos leer y sobre que bandas del buffer de destino queremos escribirlas
	 * @param dWorldTLX Posicin X en pixeles
	 * @param dWorldTLY Posicin Y en pixeles
	 * @param nWidth Ancho en pixeles
	 * @param nHeight Alto en pixeles
	 * @param bufWidth Ancho del buffer
	 * @param bufHeight Alto del buffer
	 * @throws GdalException
	 */
	public void readWindow(RasterBuf buf, BandList bandList, double dWorldTLX, double dWorldTLY, double dWorldBRX, double dWorldBRY,
            				double nWidth, double nHeight, int bufWidth, int bufHeight) throws GdalException {
		setView(dWorldTLX, dWorldTLY, dWorldBRX, dWorldBRY, bufWidth, bufHeight, new boolean[]{true, false});
		Point2D tl = worldToRaster(new Point2D.Double(dWorldTLX, dWorldTLY));
		Point2D br = worldToRaster(new Point2D.Double(dWorldBRX, dWorldBRY));

		if(gdalBands.length == 0)
			return;

		selectGdalBands(buf.getBandCount());

        int x = (int) tl.getX();
        int y = (int) tl.getY();
        int endX = (int) Math.ceil(br.getX());
        int endY = (int) Math.ceil(br.getY());

        int stpX = 0;
        int stpY = 0;

		if(bufWidth > Math.ceil(nWidth)){
			stpX = (int)(((tl.getX() - x) * bufWidth) / nWidth);
			bufWidth = (int)((Math.abs(endX - x) * bufWidth) / nWidth);
		}
		if(bufHeight > Math.ceil(nHeight)){
			stpY = (int)(((tl.getY() - y) * bufHeight) / nHeight);
			bufHeight = (int)((Math.abs(endY - y) * bufHeight) / nHeight);
		}

        nWidth = (int)Math.abs(endX - x);
        nHeight = (int)Math.abs(endY - y);

        if ((x + nWidth) > gdalBands[0].getRasterBandXSize())
        	nWidth = gdalBands[0].getRasterBandXSize() - x;

        if ((y + nHeight) > gdalBands[0].getRasterBandYSize())
        	nHeight = gdalBands[0].getRasterBandYSize() - y;

        int[] stpBuffer = new int[]{0, 0 , buf.getWidth(), buf.getHeight()};
		readData(buf, bandList, x, y, (int)nWidth, (int)nHeight, bufWidth, bufHeight, stpX, stpY, stpBuffer);
	}

	/**
	 * Lee una ventana de datos sin resampleo a partir de coordenadas en pixeles. Esta funcin es usuada por
	 * readWindow para coordenadas reales y readWindow en coordenadas pixel.
	 * @param buf Buffer donde se almacenan los datos
	 * @param bandList Lista de bandas que queremos leer y sobre que bandas del buffer de destino queremos escribirlas
	 * @param x Posicin X en pixeles
	 * @param y Posicin Y en pixeles
	 * @param w Ancho en pixeles
	 * @param yMax altura mxima de y
	 * @throws GdalException
	 */
	private void readData(RasterBuf buf, BandList bandList, int x, int y, int w, int h, int bufWidth, int bufHeight, int stpX, int stpY, int[] stepBuffer) throws GdalException {
		//TODO: FUNCIONALIDAD: Orientacin del raster. orientaion[0] para pixels en X y orientation[1] para pixels en Y
		GdalBuffer gdalBuf = null;
		for(int iBand = 0; iBand < gdalBands.length; iBand++){
			int[] drawableBands = bandList.getBufferBandToDraw(fileName, iBand);
			if(drawableBands == null || (drawableBands.length == 1 && drawableBands[0] == -1))
				continue;
			int init = (int)((bufWidth * stpY) + stpX); //Pos inicial. Desplazamos stpX pixels hacia la derecha y bajamos stpY lineas
			int pos = init;
			gdalBuf = gdalBands[iBand].readRaster(x, y, w, h, bufWidth, bufHeight, dataType);
			if(dataType == Gdal.GDT_Byte){
				for (int line = stepBuffer[1]; line < stepBuffer[3]/*buf.getHeight()*/; line++) {
					pos = (int)((bufWidth * line) + init);
					for (int col = stepBuffer[0]; col < stepBuffer[2]/*buf.getWidth()*/; col ++){
						for(int drawBands = 0; drawBands < drawableBands.length; drawBands++){ //Una misma banda del raster puede ir dibujada en varias bandas del buffer.
							buf.setElemByte(line, col, drawableBands[drawBands], gdalBuf.buffByte[pos]);
						}
						pos ++;
					}
				}
			}else if((dataType == Gdal.GDT_UInt16) || (dataType == Gdal.GDT_Int16) || (dataType == Gdal.GDT_CInt16)){
				for (int line = stepBuffer[1]; line < stepBuffer[3]; line++) {
					pos = (int)((bufWidth * line) + init);
					for (int col = stepBuffer[0]; col < stepBuffer[2]; col ++){
						for(int drawBands = 0; drawBands < drawableBands.length; drawBands++){ //Una misma banda del raster puede ir dibujada en varias bandas del buffer.
							buf.setElemShort(line, col, drawableBands[drawBands], gdalBuf.buffShort[pos]);
						}
						pos ++;
					}
				}
			}else if((dataType == Gdal.GDT_UInt32) || (dataType == Gdal.GDT_Int32) || (dataType == Gdal.GDT_CInt32)){
				for (int line = stepBuffer[1]; line < stepBuffer[3]; line++) {
					pos = (int)((bufWidth * line) + init);
					for (int col = stepBuffer[0]; col < stepBuffer[2]; col ++){
						for(int drawBands = 0; drawBands < drawableBands.length; drawBands++){ //Una misma banda del raster puede ir dibujada en varias bandas del buffer.
							buf.setElemInt(line, col, drawableBands[drawBands], gdalBuf.buffInt[pos]);
						}
						pos ++;
					}
				}
			}else if(dataType == Gdal.GDT_Float32){
				for (int line = stepBuffer[1]; line < stepBuffer[3]; line++) {
					pos = (int)((bufWidth * line) + init);
					for (int col = stepBuffer[0]; col < stepBuffer[2]; col ++){
						for(int drawBands = 0; drawBands < drawableBands.length; drawBands++){ //Una misma banda del raster puede ir dibujada en varias bandas del buffer.
							buf.setElemFloat(line, col, drawableBands[drawBands], gdalBuf.buffFloat[pos]);
						}
						pos ++;
					}
				}
			}else if(dataType == Gdal.GDT_Float64){
				for (int line = stepBuffer[1]; line < stepBuffer[3]; line++) {
					pos = (int)((bufWidth * line) + init);
					for (int col = stepBuffer[0]; col < stepBuffer[2]; col ++){
						for(int drawBands = 0; drawBands < drawableBands.length; drawBands++){ //Una misma banda del raster puede ir dibujada en varias bandas del buffer.
							buf.setElemDouble(line, col, drawableBands[drawBands], gdalBuf.buffDouble[pos]);
						}
						pos ++;
					}
				}
			}
		}
	}


	/**
	 * Lee una ventana de datos sin resampleo a partir de coordenadas reales.
	 * @param buf Buffer donde se almacenan los datos
	 * @param bandList Lista de bandas que queremos leer y sobre que bandas del buffer de destino queremos escribirlas
	 * @param dWorldTLX Posicin X superior izquierda en coord reales
	 * @param dWorldTLY Posicin Y superior izquierda en coord reales
	 * @param dWorldBRX Posicin X inferior derecha en coord reales
	 * @param dWorldBRY Posicin Y inferior derecha en coord reales
	 * @param nWidth Ancho en pixeles del buffer
	 * @param nHeight Alto en pixeles del buffer
	 * @throws GdalException
	 */
	public void readWindowWithNoData(RasterBuf buf, BandList bandList, double dWorldTLX, double dWorldTLY,double dWorldBRX, double dWorldBRY,
            				int nWidth, int nHeight) throws GdalException {
		Extent petExtent = new Extent(dWorldTLX, dWorldTLY, dWorldBRX, dWorldBRY);
		if(dWorldTLX < bBoxWithoutRot.minX)
			dWorldTLX = bBoxWithoutRot.minX;
		if(dWorldTLY > bBoxWithoutRot.maxY)
			dWorldTLY = bBoxWithoutRot.maxY;
		if(dWorldBRX > bBoxWithoutRot.maxX)
			dWorldBRX = bBoxWithoutRot.maxX;
		if(dWorldBRY < bBoxWithoutRot.minY)
			dWorldBRY = bBoxWithoutRot.minY;
		setView(dWorldTLX, dWorldTLY, dWorldBRX, dWorldBRY, nWidth, nHeight, new boolean[]{true, false});
		Point2D tl = worldToRaster(new Point2D.Double(dWorldTLX, dWorldTLY));

		if(gdalBands.length == 0)
			return;

		selectGdalBands(buf.getBandCount());

        int x = (int) tl.getX();
        int y = (int) tl.getY();

        int[] stpBuffer = new int[]{0, 0 , buf.getWidth(), buf.getHeight()};
        //Si el buffer no se ajusta al extent entonces calculamos en que posicin comienza a escribirse dentro del buffer
        //ya que lo que cae fuera sern valores NoData

        int[] wh = calcStepBuffer(petExtent, nWidth, nHeight, stpBuffer);
        if(x < 0)
        	x  = 0;
        if(y < 0)
        	y  = 0;
        readData(buf, bandList, x, y, wh[0], wh[1], wh[0], wh[1], 0, 0, stpBuffer);
	}

	/**
	 * Cuando se hace una peticin de carga de buffer la extensin pedida puede estar ajustada a la extensin del raster
	 * o no estarlo. En caso de no estarlo los pixeles del buffer que caen fuera de la extensin del raster tendrn valor
	 * de NoData. Esta funcin calcula en que pixel del buffer hay que empezar a escribir en caso de que este sea mayor
	 * que los datos a leer.
	 * @param dWorldTLX Posicin X superior izquierda en coord reales
	 * @param dWorldTLY Posicin Y superior izquierda en coord reales
	 * @param dWorldBRX Posicin X inferior derecha en coord reales
	 * @param dWorldBRY Posicin Y inferior derecha en coord reales
	 * @param nWidth Ancho en pixeles del buffer
	 * @param nHeight Alto en pixeles del buffer
	 * @return desplazamiento dentro del buffer en X e Y
	 */
	private int[] calcStepBuffer(Extent dataExtent, int nWidth, int nHeight, int[] stpBuffer){
    	Extent imageExtent = new Extent(bBoxWithoutRot.minX, bBoxWithoutRot.minY, bBoxWithoutRot.maxX, bBoxWithoutRot.maxY);
    	Extent ajustDataExtent = Utilities.calculateAdjustedView(dataExtent, imageExtent);
    	if(!Utilities.compareExtents(dataExtent, ajustDataExtent)){
    		Point2D p1 = worldToRaster(new Point2D.Double(ajustDataExtent.minX(), ajustDataExtent.maxY()));
    		Point2D p2 = worldToRaster(new Point2D.Double(ajustDataExtent.maxX(), ajustDataExtent.minY()));
    		Point2D p3 = worldToRaster(new Point2D.Double(dataExtent.minX(), dataExtent.maxY()));
    		Point2D p4 = worldToRaster(new Point2D.Double(dataExtent.maxX(), dataExtent.minY()));
    		//Ese es el ancho y alto q tendra el buffer en caso de haberse ajustado
    		int w = (int)Math.abs(Math.ceil(p2.getX()) - Math.floor(p1.getX()));
    		int h = (int)Math.abs(Math.floor(p1.getY()) - Math.ceil(p2.getY()));

    		stpBuffer[0] = (int)(p1.getX() + (-p3.getX()));
    		stpBuffer[1] = (int)(p1.getY() + (-p3.getY()));
    		stpBuffer[2] = stpBuffer[0] + w;
    		stpBuffer[3] = stpBuffer[1] + h;
    		return new int[]{w, h};
    	}
    	return new int[]{nWidth, nHeight};
	}

	/**
	 * Selecciona bandas y overview en el objeto GdalRasterBand[] para el nmero de bandas solicitado.
	 * @param nbands Nmero de bandas solicitado.
	 * @throws GdalException
	 */
	public void selectGdalBands(int nbands)throws GdalException{
		gdalBands = new GdalRasterBand[nbands];
		//Selecciona las bandas y los overviews necesarios
		gdalBands[0] = getRasterBand(1);
		for(int i = 0; i < nbands; i++)
			gdalBands[i] = gdalBands[0];

		setDataType(gdalBands[0].getRasterDataType());

		for(int i = 2; i <= nbands; i++){
			if(getRasterCount() >= i){
				gdalBands[i - 1] = getRasterBand(i);
				for(int j = i; j < nbands; j++)
					gdalBands[j] = gdalBands[i - 1];
			}
		}

		if (currentOverview > 0) {
			gdalBands[0] = gdalBands[0].getOverview(currentOverview);
			for(int i = 2; i <= nbands; i++){
				if(getRasterCount() >= i)
					gdalBands[i - 1] = gdalBands[i - 1].getOverview(currentOverview);
			}
		}
	}

	/**
	 * Lee una ventana de datos sin resampleo a partir de coordenadas en pixeles.
	 * @param buf Buffer donde se almacenan los datos
	 * @param bandList Lista de bandas que queremos leer y sobre que bandas del buffer de destino queremos escribirlas
	 * @param x Posicin X en pixeles
	 * @param y Posicin Y en pixeles
	 * @param w Ancho en pixeles
	 * @param h Alto en pixeles
	 * @throws GdalException
	 */
	public void readWindow(RasterBuf buf, BandList bandList, int x, int y, int w, int h) throws GdalException {
        GdalBuffer gdalBuf = null;
		gdalBands = new GdalRasterBand[getRasterCount()];
		isSupersampling = false;
		if(gdalBands.length == 0)
			return;

		// Selecciona las bandas
		gdalBands[0] = getRasterBand(1);
		setDataType(gdalBands[0].getRasterDataType());
		for(int iBand = 1; iBand < gdalBands.length; iBand++)
			gdalBands[iBand] = getRasterBand(iBand + 1);

		int yMax = y + h;
		readData(buf, bandList, x, y, w, yMax);
	}

	/**
	 * Lee una ventana de datos sin resampleo a partir de coordenadas en pixeles. Esta funcin es usuada por
	 * readWindow para coordenadas reales y readWindow en coordenadas pixel.
	 * @param buf Buffer donde se almacenan los datos
	 * @param bandList Lista de bandas que queremos leer y sobre que bandas del buffer de destino queremos escribirlas
	 * @param x Posicin X en pixeles
	 * @param y Posicin Y en pixeles
	 * @param w Ancho en pixeles
	 * @param yMax altura mxima de y
	 * @throws GdalException
	 */
	private void readData(RasterBuf buf, BandList bandList, int x, int y, int w, int yMax) throws GdalException {
		GdalBuffer gdalBuf = null;
		int rasterBufLine;
		for(int iBand = 0; iBand < gdalBands.length; iBand++){
			int[] drawableBands = bandList.getBufferBandToDraw(fileName, iBand);
			if(drawableBands == null || (drawableBands.length == 1 && drawableBands[0] == -1))
				continue;
			if(dataType == Gdal.GDT_Byte){
				for (int line = y; line < yMax; line++) {
					gdalBuf = gdalBands[iBand].readRaster(x, line, w, 1, w, 1, dataType);
					rasterBufLine = line - y;
					for(int drawBands = 0; drawBands < drawableBands.length; drawBands++)
						buf.setLineInBandByte(gdalBuf.buffByte, rasterBufLine, drawableBands[drawBands]);
				}
			}else if((dataType == Gdal.GDT_UInt16) || (dataType == Gdal.GDT_Int16) || (dataType == Gdal.GDT_CInt16)){
				for (int line = y; line < yMax; line++) {
					gdalBuf = gdalBands[iBand].readRaster(x, line, w, 1, w, 1, dataType);
					rasterBufLine = line - y;
					for(int drawBands = 0; drawBands < drawableBands.length; drawBands++)
						buf.setLineInBandShort(gdalBuf.buffShort, rasterBufLine, drawableBands[drawBands]);
				}
			}else if((dataType == Gdal.GDT_UInt32) || (dataType == Gdal.GDT_Int32) || (dataType == Gdal.GDT_CInt32)){
				for (int line = y; line < yMax; line++) {
					gdalBuf = gdalBands[iBand].readRaster(x, line, w, 1, w, 1, dataType);
					rasterBufLine = line - y;
					for(int drawBands = 0; drawBands < drawableBands.length; drawBands++)
						buf.setLineInBandInt(gdalBuf.buffInt, rasterBufLine, drawableBands[drawBands]);
				}
			}else if(dataType == Gdal.GDT_Float32){
				for (int line = y; line < yMax; line++) {
					gdalBuf = gdalBands[iBand].readRaster(x, line, w, 1, w, 1, dataType);
					rasterBufLine = line - y;
					for(int drawBands = 0; drawBands < drawableBands.length; drawBands++)
						buf.setLineInBandFloat(gdalBuf.buffFloat, rasterBufLine, drawableBands[drawBands]);
				}
			}else if(dataType == Gdal.GDT_Float64){
				for (int line = y; line < yMax; line++) {
					gdalBuf = gdalBands[iBand].readRaster(x, line, w, 1, w, 1, dataType);
					rasterBufLine = line - y;
					for(int drawBands = 0; drawBands < drawableBands.length; drawBands++)
						buf.setLineInBandDouble(gdalBuf.buffDouble, rasterBufLine, drawableBands[drawBands]);
				}
			}
		}
	}

	/* (non-Javadoc)
	 * @see org.cresques.io.GeoRasterFile#getData(int, int, int)
	 */
	public Object[] getData(int x, int y) {
		try {
			Object[] data = new Object[getRasterCount()];
			for(int i = 0; i < getRasterCount(); i++){
				GdalRasterBand rb = getRasterBand(i + 1);
				GdalBuffer r = rb.readRaster(x, y, 1, 1, 1, 1, dataType);
				switch(dataType){
				case 0:	break;									//Sin tipo
				case 1:	data[i] = new Integer(r.buffByte[0]); 	//Buffer byte (8)
						break;
				case 2:											//Buffer short (16)
				case 3:	data[i] = new Integer(r.buffShort[0]);	//Buffer short (16)
						break;
				case 4:											//Buffer int (32)
				case 5: data[i] = new Integer(r.buffInt[0]);	//Buffer int (32)
						break;
				case 6:	data[i] = new Float(r.buffFloat[0]);	//Buffer float (32)
						break;
				case 7:	data[i] = new Double(r.buffDouble[0]);	//Buffer double (64)
						break;
				}
			}
			return data;
		} catch (GdalException e) {
			return null;
		}
	}

	void pintaInfo() {
		try {
			//System.out.println("Origin = "+originX+","+originY);
			//System.out.println("Origin = "+this.);
			System.out.println("GeoTransform:");
			GeoTransform trans = getGeoTransform();
			for (int i=0; i<6; i++)
				System.out.println("  param["+i+"]="+trans.adfgeotransform[i]);
			System.out.println("Metadata:");
			String [] metadata = getMetadata();
			for (int i=0; i<metadata.length; i++) {
				System.out.println(metadata[i]);
			}
		} catch (GdalException e) {

		}

	}

	void pintaPaleta() {
	}

	public int getBlockSize(){
		return this.getBlockSize();
	}

	/**
	 * Obtiene el objeto que contiene los metadatos
	 */
	public Metadata getMetadataJavaObject() {
		return metadata;
	}

	/**
	 * Obtiene el flag que dice si la imagen est o no georreferenciada
	 * @return true si est georreferenciada y false si no lo est.
	 */
	public boolean isGeoreferenced() {
		return georeferenced;
	}

}



