/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.cresques.io.data;

import java.awt.geom.Rectangle2D;
import java.io.IOException;

import org.cresques.geo.ViewPortData;
import org.cresques.io.GeoRasterWriter;
import org.cresques.io.IDataWriter;
import org.cresques.px.Extent;
import org.cresques.util.Utilities;


/**
 * Clase que realiza el proceso de recorte de un raster a partir de unos datos
 * de entrada. Para realizar este recorte necesita la extensin o coordenadas 
 * pixel de la imagen, el tamao del recorte, el nmero de bandas que intervienen
 * desde la original y si se desea crear un raster por banda o todos en un mismo 
 * raster. Como fuente de datos de entrada estar el grid que a travs de un proceso
 * que muestra el incremento de la operacin crea el/los recorte/s. Debemos indicarle
 * tambin el directorio donde queremos crear el/los raster y el nombre del mismo.
 * 
 * En caso de generar varios (uno por banda) pondr el mismo nomnbre a todos pero 
 * numerandolos (raster_1.tif, raster_2.tif, ...)
 *
 * @author Nacho Brodin (nachobrodin@gmail.com)
 */
public class CutRasterProcess extends Thread {
	
	/**
	 * Escritores para salvar los raster a disco. Uno por raster a salvar.
	 */
	private GeoRasterWriter[] 			writers = null;
	/**
	 * Bandas que se salvan y en el orden que se hace. Cada posicin del array es 
	 * una banda que se salvar y el elemento que contiene el nmero de banda del 
	 * grid que va en esa posicin.
	 */
	private int[] 						bands = null;
	/**
	 * Datos fuente
	 */
	private Grid						grid = null;
	/**
	 * Nombre de el/los ficheros donde se salva. 
	 */
	private volatile String[]			fileNames = null;
	
	private ViewPortData 				vpData = null;
	/**
	 * Rasterizador de bandas
	 */
	private RasterizeGrid[]				rasterizer = null;
	/**
	 * Clase que instancia a esta y que pone en marcha el thread para generar los recortes.
	 */
	private ICutRaster					parent = null;
	/**
	 * Ancho y alto del grid resultante
	 */
	//private int 						width = 0, height = 0;
	/**
	 * Informa de si hay un fichero por banda o todas las bandas en un fichero. True en el primer caso
	 * y false en el segundo.
	 */
	private boolean 					aFilePerBand = false;
	/**
	 * Nombre del fichero destino. En caso de crearse un fichero por banda, se numerarn. 
	 * (raster_1.tif, raster_2.tif, ...)
	 */
	private String						fileName = null;
	
	/**
	 * Constructor cuando la seleccin se hace por coordenadas pixel
	 * @param pixelCoord Coordenadas pixeles que representan los margenes del recorte
	 * @param bands Lista de bandas a salvar. Cada posicin del array es una banda que se
	 * salvar y el elemento que contiene el nmero de banda del grid que va en esa posicin
	 * por lo que si tenemos un grid de 3 bandas y queremos hacer un recorte con solo la 
	 * banda tres pasaremos un array {3}
	 * @param grid Grid con los datos fuente
	 * @param fileName Nombre del fichero destino. En caso de crearse un fichero por banda, 
	 * se numerarn. (raster_1.tif, raster_2.tif, ...)
	 * @param aFilePerBand Variable que dice si hay un fichero por banda o todas las bandas 
	 * van en un fichero. En el primer caso valdr true y en el segundo false.
	 * @param w Ancho en pixeles del raster resultante
	 * @param h Alto en pixeles del raster resultante
	 * @param parent Objeto que implementa el interfaz ITailRaster que tiene los mtodos que
	 * son llamados cuando se finaliza el proceso de recortado. 
	 */
	public CutRasterProcess(int[] pixelCoord, int[] bands, Grid grid, String fileName, boolean aFilePerBand, int w, int h, ICutRaster parent){
		this.bands = bands;
		this.grid = grid;
		this.parent = parent;
		this.aFilePerBand = aFilePerBand;
		this.fileName = fileName;
		
		grid.clearDrawableBand();
		grid.addDrawableBands(bands);
		grid.setAreaOfInterest(pixelCoord[0], pixelCoord[1], w, h);
						
		//Hallamos primero las coordenadas del mapa que corresponden a esos pixeles y calculamos con ellos el extent
		Rectangle2D extent = Utilities.getMapRectFromPxRect(	grid.getExtent().toRectangle2D(), 
				grid.getWidth(), grid.getHeight(),
				new Rectangle2D.Double(pixelCoord[0], pixelCoord[1], w, h));
		
		vpData = new ViewPortData(null, new Extent(extent), null);
	
		//createWriters(grid.getRasterBuf());
	}
	
	/**
	 * Constructor cuando la seleccin se hace por coordenadas reales
	 * @param mapCoord Coordenadas del mundo real que representan los margenes del recorte
	 * @param bands Lista de bandas a salvar. Cada posicin del array es una banda que se
	 * salvar y el elemento que contiene el nmero de banda del grid que va en esa posicin
	 * por lo que si tenemos un grid de 3 bandas y queremos hacer un recorte con solo la 
	 * banda tres pasaremos un array {3}
	 * @param grid Grid con los datos fuente
	 * @param fileName Nombre del fichero destino. En caso de crearse un fichero por banda, 
	 * se numerarn. (raster_1.tif, raster_2.tif, ...)
	 * @param aFilePerBand Variable que dice si hay un fichero por banda o todas las bandas 
	 * van en un fichero. En el primer caso valdr true y en el segundo false.
	 * @param w Ancho en pixeles del raster resultante
	 * @param h Alto en pixeles del raster resultante
	 * @param parent Objeto que implementa el interfaz ITailRaster que tiene los mtodos que
	 * son llamados cuando se finaliza el proceso de recortado.
	 */
	public CutRasterProcess(double[] mapCoord, int[] bands, Grid grid, String fileName, boolean aFilePerBand, int w, int h, ICutRaster parent){
		this.bands = bands;
		this.grid = grid;
		this.parent = parent;
		this.aFilePerBand = aFilePerBand;
		this.fileName = fileName;
		
		grid.clearDrawableBand();
		grid.addDrawableBands(bands);
		grid.setAreaOfInterest(mapCoord[0], mapCoord[1], Math.abs(mapCoord[2] - mapCoord[0]), Math.abs(mapCoord[3] - mapCoord[1]));
		
		//Calculamos el Extent con las coordenadas del mapa pasadas
		vpData = new ViewPortData(null, new Extent(mapCoord[0], mapCoord[1], mapCoord[2], mapCoord[3]), null);
		
		//createWriters(grid.getRasterBuf());
	}
	
	/**
	 * Creador de writers. Genera uno por fichero a salvar. Si hay un solo fichero se pone
	 * el nombre pasado por parmetro, si hay uno por bandas ser el nombre ms un nmero de
	 * secuencia (raster_1.tif, raster_2.tif, ...)
	 * @param rasterBuf Buffer desde donde se obtienen los datos.
	 */
	public String[] createWriters(RasterBuf rasterBuf, int w, int h){
		if(aFilePerBand){
			rasterizer = new RasterizeGrid[bands.length];
			fileNames = new String[bands.length];
			writers = new GeoRasterWriter[bands.length];
			
			for(int iFile = 0; iFile < bands.length; iFile ++){
				rasterizer[iFile] = new RasterizeGrid(rasterBuf, iFile);
				fileNames[iFile] = fileName + "_" + iFile + ".tif";
				writers[iFile] = GeoRasterWriter.getWriter((IDataWriter)rasterizer[iFile], fileNames[iFile], RasterizeGrid.BLOCKSIZE, 
						1, vpData, 0, w, h, grid.getDataType());
			}
		}else{
			rasterizer = new RasterizeGrid[1];
			rasterizer[0] = new RasterizeGrid(rasterBuf, bands);
			fileNames = new String[1];
			fileNames[0] = fileName + ".tif";
			writers = new GeoRasterWriter[1];
			writers[0] = GeoRasterWriter.getWriter((IDataWriter)rasterizer[0], fileNames[0], RasterizeGrid.BLOCKSIZE, 
					bands.length, vpData, 0, w, h, grid.getDataType());
		}
		return fileNames;
	}
		
	/**
	 * Proceso que escribe la/las imagen/es
	 */
	public synchronized void run(){
		try{
			for(int i = 0; i < writers.length; i ++){
				writers[i].dataWrite();
				writers[i].writeClose(); 	
			}
		}catch(IOException ev){
			return;
		}
		parent.cutFinalize(fileNames);
	}
}
