/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2006 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 */
package org.cresques.io.data;

import java.io.IOException;

import org.cresques.filter.IRaster;
import org.cresques.io.GeoRasterWriter;
import org.cresques.io.IDataWriter;
import org.cresques.px.Extent;


/**
 * Rectangulo de pixeles. Para cada tipo de datos java hay un buffer distinto donde cada elemento es
 * accedido de la siguiente forma: [banda][fila][columna]
 * m[1][2][0] = cte;-> Sustituye el elemento de la fila 2 de la banda 1 columna 0
 * m[1][0] = array; -> Sustituye la fila 0 de la banda 1 
 * m[0] = matriz cuadrada; -> Sustituye la banda entera.
 * 
 * @author Luis W. Sevilla (sevilla_lui@gva.es)
 */
public class RasterBuf implements IRaster {
    private byte[][][] 		byteBuf;
    private short[][][] 	shortBuf;
    private int[][][] 		intBuf;
    private float[][][]		floatBuf;
    private double[][][]	doubleBuf;
    
    private int 			width;
    private int 			height;
    private int 			nBands;
    private int 			dataType;
	public double 			noDataValue = -99999;

    /**
     * Constructor
     */
    public RasterBuf() {
    	
    }
    
    /**
     * Constructor
     * @param dataType Tipo de dato
     * @param width Ancho
     * @param height Alto
     * @param bandNr Banda
     * @param orig
     */
    public RasterBuf(int dataType, int width, int height, int bandNr, boolean malloc) {
    	if(malloc)
    		malloc(dataType, width, height, bandNr);
    	else
    		loadVariables(dataType, width, height, bandNr);
    }

    /**
     * Reserva de memoria para el rasterbuf
     * @param dataType Tipo de dato
     * @param width Ancho
     * @param height Alto
     * @param bandNr Banda
     * @param orig
     */
    public void malloc(int dataType, int width, int height, int bandNr) {
    	this.dataType = dataType;
        this.width = width;
        this.height = height;
        this.nBands = bandNr;

        if (dataType == TYPE_BYTE) {
            byteBuf = new byte[bandNr][height][width];
        } else if ((dataType == TYPE_SHORT) | (dataType == TYPE_USHORT)) {
            shortBuf = new short[bandNr][height][width];
        } else if (dataType == TYPE_INT) {
            intBuf = new int[bandNr][height][width];
        } else if (dataType == TYPE_FLOAT) {
            floatBuf = new float[bandNr][height][width];
        } else if (dataType == TYPE_DOUBLE) {
            doubleBuf = new double[bandNr][height][width];
        }

    }

    /**
     * Carga las variables del rasterBuf
     * @param dataType Tipo de dato
     * @param width Ancho
     * @param height Alto
     * @param bandNr Banda
     * @param orig
     */
    private void loadVariables(int dataType, int width, int height, int bandNr) {
    	this.dataType = dataType;
        this.width = width;
        this.height = height;
        this.nBands = bandNr;

        if (dataType == TYPE_BYTE) {
            byteBuf = new byte[bandNr][height][];
        } else if ((dataType == TYPE_SHORT) | (dataType == TYPE_USHORT)) {
            shortBuf = new short[bandNr][height][];
        } else if (dataType == TYPE_INT) {
            intBuf = new int[bandNr][height][];
        } else if (dataType == TYPE_FLOAT) {
            floatBuf = new float[bandNr][height][];
        } else if (dataType == TYPE_DOUBLE) {
            doubleBuf = new double[bandNr][height][];
        }

    }

    
    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRaster#getWidth()
     */
    public int getWidth() {
        return width;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRaster#getHeight()
     */
    public int getHeight() {
        return height;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRaster#getBandNr()
     */
    public int getBandCount() {
        return nBands;
    }

    /* (non-Javadoc)
     * @see org.cresques.io.raster.IRaster#getDataType()
     */
    public int getDataType() {
        return dataType;
    }

    /**
     * Obtiene el tamao del tipo de dato en bytes
     * @return Tipo de dato
     */
    public int getDataSize() {
        if (dataType == TYPE_BYTE) {
            return 1;
        } else if ((dataType == TYPE_SHORT) | (dataType == TYPE_USHORT)) {
            return 2;
        } else if (dataType == TYPE_INT) {
            return 4;
        }else if (dataType == TYPE_FLOAT) {
            return 8;
        }else if (dataType == TYPE_DOUBLE) {
            return 16;
        }

        return 0;
    }

    /**
     * Obtiene el tamao del buffer
     * @return tamao del buffer
     */
    public int sizeof() {
        return getDataSize() * width * height * nBands;
    }

    //***********************************************
    //Obtiene una linea de datos con todas las bandas
    
    public byte[][] getLineByte(int line) {
    	byte[][] r = new byte[nBands][];
    	for(int iBand = 0; iBand < nBands; iBand ++)
    		r[iBand] = byteBuf[iBand][line];
    	return r;
    }

    public short[][] getLineShort(int line) {
    	short[][] r = new short[nBands][];
    	for(int iBand = 0; iBand < nBands; iBand ++)
    		r[iBand] = shortBuf[iBand][line];
    	return r;
    }

    public int[][] getLineInt(int line) {
    	int[][] r = new int[nBands][];
    	for(int iBand = 0; iBand < nBands; iBand ++)
    		r[iBand] = intBuf[iBand][line];
    	return r;
    }
        
    public float[][] getLineFloat(int line) {
    	float[][] r = new float[nBands][];
    	for(int iBand = 0; iBand < nBands; iBand ++)
    		r[iBand] = floatBuf[iBand][line];
    	return r;
    }
    
    public double[][] getLineDouble(int line) {
    	double[][] r = new double[nBands][];
    	for(int iBand = 0; iBand < nBands; iBand ++)
    		r[iBand] = doubleBuf[iBand][line];
    	return r;
    }

    //***********************************************
    //Asigna una linea de datos a todas las bandas
    
    public void setLineByte(byte[][] data, int line) {
    	for(int iBand = 0; iBand < nBands; iBand ++)
    		byteBuf[iBand][line] = data[iBand];
    }

    public void setLineShort(short[][] data, int line) {
    	for(int iBand = 0; iBand < nBands; iBand ++)
    		shortBuf[iBand][line] = data[iBand];
    }

    public void setLineInt(int[][] data, int line) {
    	for(int iBand = 0; iBand < nBands; iBand ++)
    		intBuf[iBand][line] = data[iBand];
    }
        
    public void setLineFloat(float[][] data, int line) {
    	for(int iBand = 0; iBand < nBands; iBand ++)
    		floatBuf[iBand][line] = data[iBand];
    }
    
    public void setLineDouble(double[][] data, int line) {
    	for(int iBand = 0; iBand < nBands; iBand ++)
    		doubleBuf[iBand][line] = data[iBand];
    }
    
    //***********************************************
    //Obtiene una linea de datos de una banda
    
    public byte[] getLineFromBandByte(int line, int band) {
        return byteBuf[band][line];
    }

    public short[] getLineFromBandShort(int line, int band) {
        return shortBuf[band][line];
    }

    public int[] getLineFromBandInt(int line, int band) {
        return intBuf[band][line];
    }
        
    public float[] getLineFromBandFloat(int line, int band) {
        return floatBuf[band][line];
    }
    
    public double[] getLineFromBandDouble(int line, int band) {
        return doubleBuf[band][line];
    }
        
    //***********************************************
    //Asigna una linea de datos a una banda
    
    public void setLineInBandByte(byte[] data, int line, int band) {
        byteBuf[band][line] = data;
    }

    public void setLineInBandShort(short[] data, int line, int band) {
        shortBuf[band][line] = data;
    }

    public void setLineInBandInt(int[] data, int line, int band) {
        intBuf[band][line] = data;
    }
        
    public void setLineInBandFloat(float[] data, int line, int band) {
        floatBuf[band][line] = data;
    }
    
    public void setLineInBandDouble(double[] data, int line, int band) {
        doubleBuf[band][line] = data;
    }
    
    //***********************************************    
    //Obtiene un elemento de la matriz
    
    public byte getElemByte(int line, int col, int band) {
        return byteBuf[band][line][col];
    }
    
    public short getElemShort(int line, int col, int band) {
        return shortBuf[band][line][col];
    }
    
    public int getElemInt(int line, int col, int band) {
        return intBuf[band][line][col];
    }
    
    public float getElemFloat(int line, int col, int band) {
        return floatBuf[band][line][col];
    }
    
    public double getElemDouble(int line, int col, int band) {
        return doubleBuf[band][line][col];
    }
    
    //**********************************************    
    //Asigna un elemento de la matriz
    
    public void setElemByte(int line, int col, int band, byte data) {
        byteBuf[band][line][col] = data;
    }
    
    public void setElemShort(int line, int col, int band, short data) {
        shortBuf[band][line][col] = data;
    }
    
    public void setElemInt(int line, int col, int band, int data) {
        intBuf[band][line][col] = data;
    }
    
    public void setElemFloat(int line, int col, int band, float data) {
        floatBuf[band][line][col] = data;
    }
    
    public void setElemDouble(int line, int col, int band, double data) {
        doubleBuf[band][line][col] = data;
    }
    
    //***********************************************
    //Copia un elemento de todas la bandas en el buffer pasado por parmetro
    
    public void getElemByte(int line, int col, byte[] data) {
    	for (int iBand = 0; (iBand < nBands && iBand < data.length); iBand++)
            data[iBand] = byteBuf[iBand][line][col];
    }
    
    public void getElemShort(int line, int col, short[] data) {
    	for (int iBand = 0; (iBand < nBands && iBand < data.length); iBand++)
            data[iBand] = shortBuf[iBand][line][col];
    }
    
    public void getElemInt(int line, int col, int[] data) {
    	for (int iBand = 0; (iBand < nBands && iBand < data.length); iBand++)
            data[iBand] = intBuf[iBand][line][col];
    }  
    
    public void getElemFloat(int line, int col, float[] data) {
    	for (int iBand = 0; (iBand < nBands && iBand < data.length); iBand++)
            data[iBand] = floatBuf[iBand][line][col];
    }
    
    public void getElemDouble(int line, int col, double[] data) {
    	for (int iBand = 0; (iBand < nBands && iBand < data.length); iBand++)
            data[iBand] = doubleBuf[iBand][line][col];
    }

    //***********************************************
    //Asigna un elemento a todas la bandas en el buffer pasado por parmetro
    
    public void setElemByte(int line, int col, byte[] data) {
    	for (int iBand = 0; (iBand < nBands && iBand < data.length); iBand++)
            byteBuf[iBand][line][col] = data[iBand];
    }
    
    public void setElemShort(int line, int col, short[] data) {
    	for (int iBand = 0; (iBand < nBands && iBand < data.length); iBand++)
            shortBuf[iBand][line][col] = data[iBand];
    }
    
    public void setElemInt(int line, int col, int[] data) {
    	for (int iBand = 0; (iBand < nBands && iBand < data.length); iBand++)
            intBuf[iBand][line][col] = data[iBand];
    }

    public void setElemFloat(int line, int col, float[] data) {
    	for (int iBand = 0; (iBand < nBands && iBand < data.length); iBand++)
            floatBuf[iBand][line][col] = data[iBand];
    }
    
    public void setElemDouble(int line, int col, double[] data) {
    	for (int iBand = 0; (iBand < nBands && iBand < data.length); iBand++)
            doubleBuf[iBand][line][col] = data[iBand];
    }

    //***********************************************
    //Obtiene una banda entera
    
    public byte[][] getBandByte(int band) {
        return byteBuf[band];
    }

    public short[][] getBandShort(int band) {
        return shortBuf[band];
    }

    public int[][] getBandInt(int band) {
        return intBuf[band];
    }
        
    public float[][] getBandFloat(int band) {
        return floatBuf[band];
    }
    
    public double[][] getBandDouble(int band) {
        return doubleBuf[band];
    }

    //***********************************************
    //Sustituye una banda entera
    
    public void getBandByte(int band, byte[][] data) {
        byteBuf[band] = data;
    }

    public void getBandShort(int band, short[][] data) {
        shortBuf[band] = data;
    }

    public void getBandInt(int band, int[][] data) {
        intBuf[band] = data;
    }
        
    public void getBandFloat(int band, float[][] data) {
        floatBuf[band] = data;
    }
    
    public void getBandDouble(int band, double[][] data) {
        doubleBuf[band] = data;
    }
    
    //***********************************************
    //Crea un buffer banda inicializado con el valor pasado por parmetro

    public static byte[][] createByteBand(int width, int height, byte defaultValue){
    	byte[][] band = new byte[height][width];
    	if(defaultValue != 0){
	    	for(int line = 0; line < height; line ++)
	    		for(int col = 0; col < width; col ++)
	    			band[line][col] = defaultValue;
    	}
    	return band;
    }
    
    public static short[][] createShortBand(int width, int height, short defaultValue){
    	short[][] band = new short[height][width];
    	if(defaultValue != 0){
	    	for(int line = 0; line < height; line ++)
	    		for(int col = 0; col < width; col ++)
	    			band[line][col] = defaultValue;
    	}
    	return band;
    }
    
    public static int[][] createIntBand(int width, int height, int defaultValue){
    	int[][] band = new int[height][width];
    	if(defaultValue != 0){
	    	for(int line = 0; line < height; line ++)
	    		for(int col = 0; col < width; col ++)
	    			band[line][col] = defaultValue;
    	}
    	return band;
    }
    
    public static float[][] createFloatBand(int width, int height, float defaultValue){
    	float[][] band = new float[height][width];
    	if(defaultValue != 0){
	    	for(int line = 0; line < height; line ++)
	    		for(int col = 0; col < width; col ++)
	    			band[line][col] = defaultValue;
    	}
    	return band;
    }
    
    public static double[][] createDoubleBand(int width, int height, double defaultValue){
    	double[][] band = new double[height][width];
    	if(defaultValue != 0){
	    	for(int line = 0; line < height; line ++)
	    		for(int col = 0; col < width; col ++)
	    			band[line][col] = defaultValue;
    	}
    	return band;
    }
    
    //***********************************************
    //Aade una banda entera. Los datos son asignados por referencia

    public void addBandByte(int pos, byte[][] data) {
    	if(pos < 0)
    		return;
    	byte[][][] tmp = null;
    	if(pos >= byteBuf.length){
    		tmp = new byte[pos + 1][][];
    		for(int iBand = 0; iBand < byteBuf.length; iBand ++)
    			tmp[iBand] = byteBuf[iBand];
    		tmp[pos] = data;
    	}else{
    		tmp = new byte[byteBuf.length + 1][][];
    		for(int iBand = 0; iBand < pos; iBand ++)
    			tmp[iBand] = byteBuf[iBand];
    		tmp[pos] = data;
    		for(int iBand = pos + 1; iBand < byteBuf.length; iBand ++)
    			tmp[iBand] = byteBuf[iBand - 1];
    	}
    	nBands++;
    	byteBuf = tmp;
    }
    
    public void addBandShort(int pos, short[][] data) {
    	if(pos < 0)
    		return;
    	short[][][] tmp = null;
    	if(pos >= shortBuf.length){
    		tmp = new short[pos + 1][][];
    		for(int iBand = 0; iBand < shortBuf.length; iBand ++)
    			tmp[iBand] = shortBuf[iBand];
    		tmp[pos] = data;
    	}else{
    		tmp = new short[shortBuf.length + 1][][];
    		for(int iBand = 0; iBand < pos; iBand ++)
    			tmp[iBand] = shortBuf[iBand];
    		tmp[pos] = data;
    		for(int iBand = pos + 1; iBand < shortBuf.length; iBand ++)
    			tmp[iBand] = shortBuf[iBand - 1];
    	}
    	nBands++;
    	shortBuf = tmp;
    }
    
    public void addBandInt(int pos, int[][] data) {
    	if(pos < 0)
    		return;
    	int[][][] tmp = null;
    	if(pos >= intBuf.length){
    		tmp = new int[pos + 1][][];
    		for(int iBand = 0; iBand < intBuf.length; iBand ++)
    			tmp[iBand] = intBuf[iBand];
    		tmp[pos] = data;
    	}else{
    		tmp = new int[intBuf.length + 1][][];
    		for(int iBand = 0; iBand < pos; iBand ++)
    			tmp[iBand] = intBuf[iBand];
    		tmp[pos] = data;
    		for(int iBand = pos + 1; iBand < intBuf.length; iBand ++)
    			tmp[iBand] = intBuf[iBand - 1];
    	}
    	nBands++;
    	intBuf = tmp;
    }
    
    public void addBandFloat(int pos, float[][] data) {
    	if(pos < 0)
    		return;
    	float[][][] tmp = null;
    	if(pos >= floatBuf.length){
    		tmp = new float[pos + 1][][];
    		for(int iBand = 0; iBand < floatBuf.length; iBand ++)
    			tmp[iBand] = floatBuf[iBand];
    		tmp[pos] = data;
    	}else{
    		tmp = new float[floatBuf.length + 1][][];
    		for(int iBand = 0; iBand < pos; iBand ++)
    			tmp[iBand] = floatBuf[iBand];
    		tmp[pos] = data;
    		for(int iBand = pos + 1; iBand < floatBuf.length; iBand ++)
    			tmp[iBand] = floatBuf[iBand - 1];
    	}
    	nBands++;
    	floatBuf = tmp;
    }
    
    public void addBandDouble(int pos, double[][] data) {
    	if(pos < 0)
    		return;
    	double[][][] tmp = null;
    	if(pos >= doubleBuf.length){
    		tmp = new double[pos + 1][][];
    		for(int iBand = 0; iBand < doubleBuf.length; iBand ++)
    			tmp[iBand] = doubleBuf[iBand];
    		tmp[pos] = data;
    	}else{
    		tmp = new double[doubleBuf.length + 1][][];
    		for(int iBand = 0; iBand < pos; iBand ++)
    			tmp[iBand] = doubleBuf[iBand];
    		tmp[pos] = data;
    		for(int iBand = pos + 1; iBand < doubleBuf.length; iBand ++)
    			tmp[iBand] = doubleBuf[iBand - 1];
    	}
    	nBands++;
    	doubleBuf = tmp;
    }
    
    //***********************************************
    //Reemplaza una banda entera. Los datos son reemplazados por referencia
    
    public void replaceBandByte(int pos, byte[][] data) {
    	if(pos >= byteBuf.length)
    		return;
    	byteBuf[pos] = data;
    }
    
    public void replaceBandShort(int pos, short[][] data) {
    	if(pos >= shortBuf.length)
    		return;
    	shortBuf[pos] = data;
    }
    
    public void replaceBandInt(int pos, int[][] data) {
    	if(pos >= intBuf.length)
    		return;
    	intBuf[pos] = data;
    }
    
    public void replaceBandFloat(int pos, float[][] data) {
    	if(pos >= floatBuf.length)
    		return;
    	floatBuf[pos] = data;
    }
    
    public void replaceBandDouble(int pos, double[][] data) {
    	if(pos >= doubleBuf.length)
    		return;
    	doubleBuf[pos] = data;
    }
    
    /**
     * Replica la banda de una posicin sobre otra. Si la banda de destino no existe
     * se crea nueva. Si la posicin de la banda de destino est intercalada entre bandas 
     * que ya existen las otras se desplazan hacia abajo, NO se machacan los datos de ninguna.
     * Los datos se replican por referencia por lo que al modificar la banda original las
     * del resto quedarn afectadas.
     * @param orig. Posicin de la banda de origen. 
     * @param dest. Posicin de la banda destino
     */   
    public void replicateBand(int orig, int dest){
    	copyBand(this, orig, dest);
    }
    
    /**
     * Copia una banda de una imagen desde una posicin sobre otra en la imagen actual.
     *  Si la banda de destino no existe
     * se crea nueva. Si la posicin de la banda de destino est intercalada entre bandas 
     * que ya existen las otras se desplazan hacia abajo, NO se machacan los datos de ninguna.
     * Los datos se replican por referencia por lo que al modificar la banda original las
     * del resto quedarn afectadas.
     * @param orig. Posicin de la banda de origen. 
     * @param dest. Posicin de la banda destino
     */   
    public void copyBand(RasterBuf from, int orig, int dest){
    	switch(getDataType()){
    	case RasterBuf.TYPE_BYTE: 	if(orig >= byteBuf.length)
										return;
									addBandByte(dest, from.getBandByte(orig));
									break;
    	case RasterBuf.TYPE_SHORT: 	if(orig >= shortBuf.length)
    									return;
    								addBandShort(dest, from.getBandShort(orig));
    								break;
    	case RasterBuf.TYPE_INT: 	if(orig >= intBuf.length)
										return;
									addBandInt(dest, from.getBandInt(orig));
									break;
    	case RasterBuf.TYPE_FLOAT: 	if(orig >= floatBuf.length)
										return;
									addBandFloat(dest, from.getBandFloat(orig));
									break;
    	case RasterBuf.TYPE_DOUBLE:	if(orig >= doubleBuf.length)
										return;
									addBandDouble(dest, from.getBandDouble(orig));
									break;
    	}
    }
    /**
     * Convierte un tipo de dato a cadena
     * @param type Tipo de dato
     * @return cadena  que representa el tipo de dato
     */
    public static String typesToString(int type) {
        switch (type) {
        case RasterBuf.TYPE_IMAGE:
            return new String("Image");

        case RasterBuf.TYPE_BYTE:
            return new String("Byte");

        case RasterBuf.TYPE_DOUBLE:
            return new String("Double");

        case RasterBuf.TYPE_FLOAT:
            return new String("Float");

        case RasterBuf.TYPE_INT:
        	return new String("Integer");
        	
        case RasterBuf.TYPE_USHORT:
        case RasterBuf.TYPE_SHORT:
            return new String("Short");
        }

        return null;
    }
    
    /**
     * Ajusta el raster al ancho y alto solicitado por el vecino ms cercano.
     * @param w Nuevo ancho
     * @param h Nuevo alto
     */
    public RasterBuf adjustRasterNearestNeighbour(int w, int h, int[] bands){
    	double stepX = (double)w / (double)width;
    	double stepY = (double)h / (double)height;
    	RasterBuf rasterBuf = new RasterBuf(getDataType(), w, h, getBandCount(), true);
    	
    	//Si bands == null las bandas a copiar son todas las de la imagen
    	if(bands == null){
    		bands = new int[rasterBuf.getBandCount()];
    		for(int iBand = 0; iBand < rasterBuf.getBandCount(); iBand ++)
    			bands[iBand] = iBand;
    	}
    	
    	switch (dataType) {
        case RasterBuf.TYPE_IMAGE:
                    
        case RasterBuf.TYPE_BYTE:
        	for(int iBand = 0; iBand < bands.length; iBand ++){
        		if(w <= width){ //submuestreo
        			for(int iRow = 0; iRow < height; iRow ++)
        				for(int iCol = 0; iCol < width; iCol ++)
        					rasterBuf.setElemByte((int)(iRow * stepY), (int)(iCol * stepX), bands[iBand], getElemByte(iRow, iCol, iBand));
        		}else{ //supermuestreo
        			for(int iRow = 0; iRow < h; iRow ++)
        				for(int iCol = 0; iCol < w; iCol ++)
        					rasterBuf.setElemByte(iRow, iCol, bands[iBand], getElemByte((int)(iRow / stepY), (int)(iCol / stepX), iBand));
        		}
        	}
        	break;
        case RasterBuf.TYPE_DOUBLE:
        	for(int iBand = 0; iBand < bands.length; iBand ++){
        		if(w <= width){ //submuestreo
        			for(int iRow = 0; iRow < height; iRow ++)
        				for(int iCol = 0; iCol < width; iCol ++)
        					rasterBuf.setElemDouble((int)(iRow * stepY), (int)(iCol * stepX), bands[iBand], getElemDouble(iRow, iCol, iBand));
        		}else{ //supermuestreo
        			for(int iRow = 0; iRow < h; iRow ++)
        				for(int iCol = 0; iCol < w; iCol ++)
        					rasterBuf.setElemDouble(iRow, iCol, bands[iBand], getElemDouble((int)(iRow / stepY), (int)(iCol / stepX), iBand));
        		}
        	}
        	break;
        case RasterBuf.TYPE_FLOAT:
        	for(int iBand = 0; iBand < bands.length; iBand ++){
        		if(w <= width){ //submuestreo
        			for(int iRow = 0; iRow < height; iRow ++)
        				for(int iCol = 0; iCol < width; iCol ++)
        					rasterBuf.setElemFloat((int)(iRow * stepY), (int)(iCol * stepX), bands[iBand], getElemFloat(iRow, iCol, iBand));
        		}else{ //supermuestreo
        			for(int iRow = 0; iRow < h; iRow ++)
        				for(int iCol = 0; iCol < w; iCol ++)
        					rasterBuf.setElemFloat(iRow, iCol, bands[iBand], getElemFloat((int)(iRow / stepY), (int)(iCol / stepX), iBand));
        		}
        	}
        	break;
        case RasterBuf.TYPE_INT:
        	for(int iBand = 0; iBand < bands.length; iBand ++){
        		if(w <= width){ //submuestreo
        			for(int iRow = 0; iRow < height; iRow ++)
        				for(int iCol = 0; iCol < width; iCol ++)
        					rasterBuf.setElemInt((int)(iRow * stepY), (int)(iCol * stepX), bands[iBand], getElemInt(iRow, iCol, iBand));
        		}else{ //supermuestreo
        			for(int iRow = 0; iRow < h; iRow ++)
        				for(int iCol = 0; iCol < w; iCol ++)
        					rasterBuf.setElemInt(iRow, iCol, bands[iBand], getElemInt((int)(iRow / stepY), (int)(iCol / stepX), iBand));
        		}
        	}
        	break;
        case RasterBuf.TYPE_USHORT:
        case RasterBuf.TYPE_SHORT:
        	for(int iBand = 0; iBand < bands.length; iBand ++){
        		if(w <= width){ //submuestreo
        			for(int iRow = 0; iRow < height; iRow ++)
        				for(int iCol = 0; iCol < width; iCol ++)
        					rasterBuf.setElemShort((int)(iRow * stepY), (int)(iCol * stepX), bands[iBand], getElemShort(iRow, iCol, iBand));
        		}else{ //supermuestreo
        			for(int iRow = 0; iRow < h; iRow ++)
        				for(int iCol = 0; iCol < w; iCol ++)
        					rasterBuf.setElemShort(iRow, iCol, bands[iBand], getElemShort((int)(iRow / stepY), (int)(iCol / stepX), iBand));
        		}
        	}
        	break;
        }
    	return rasterBuf; 
    }
    
    /**
     * Salva a un formato soportado por GeoRasterWriter. El formato de destino 
     * est soportado por la extensin del fichero seleccionado en el parmetro path.
     * @param path Ruta al fichero de destino
     * @param ex	Extent
     * @throws IOException
     */
    public void save(String path, Extent ex) throws IOException {
		IDataWriter dataWriter = new WriterBufferServer(this);
		GeoRasterWriter grw = GeoRasterWriter.getWriter(dataWriter, 
														path, 
														height, //Tamao de cada bloque ledo. 
															//Si ponemos el alto de la imagen la leer toda de golpe 
														getBandCount(), 
														ex, 
														0, 
														width, 
														height, 
														getDataType());
		grw.dataWrite();
		grw.writeClose();
		
	}
    
    /*
     *  (non-Javadoc)
     * @see org.gvsig.fmap.driver.IBuffer#getNoDataValue()
     */
    public double getNoDataValue(){
    	return noDataValue;
    }
    
    /*
     *  (non-Javadoc)
     * @see org.gvsig.fmap.driver.IBuffer#getByteNoDataValue()
     */
    public byte getByteNoDataValue(){
    	return (byte)noDataValue;
    }
    
    /*
     *  (non-Javadoc)
     * @see org.gvsig.fmap.driver.IBuffer#getShortNoDataValue()
     */
    public short getShortNoDataValue(){
    	return (short)noDataValue;
    }
    
    /*
     *  (non-Javadoc)
     * @see org.gvsig.fmap.driver.IBuffer#getIntNoDataValue()
     */
    public int getIntNoDataValue(){
    	return (int)noDataValue;
    }
    
    /*
     *  (non-Javadoc)
     * @see org.gvsig.fmap.driver.IBuffer#getFloatNoDataValue()
     */
    public float getFloatNoDataValue(){
    	return (float)noDataValue;
    }
    
    /*
     *  (non-Javadoc)
     * @see org.gvsig.fmap.driver.IBuffer#setNoDataValue(double)
     */
    public void setNoDataValue(double nd){
    	noDataValue = nd;
    }
    
    public void assign(int band, byte value) {
    	for(int line = 0; line < height; line ++)
    		for(int col = 0; col < width; col ++)
    			byteBuf[band][line][col] = value;
    }
    
    public void assign(int band, short value) {
    	for(int line = 0; line < height; line ++)
    		for(int col = 0; col < width; col ++)
    			shortBuf[band][line][col] = value;
    }
    
    public void assign(int band, int value) {
    	for(int line = 0; line < height; line ++)
    		for(int col = 0; col < width; col ++)
    			intBuf[band][line][col] = value;
    }

    public void assign(int band, float value) {
    	for(int line = 0; line < height; line ++)
    		for(int col = 0; col < width; col ++)
    			floatBuf[band][line][col] = value;
    }
    
    public void assign(int band, double value) {
    	for(int line = 0; line < height; line ++)
    		for(int col = 0; col < width; col ++)
    			doubleBuf[band][line][col] = value;
    }
    
    public void setToNoData(){
    	if (dataType == TYPE_BYTE) {
    		for (int iBand = 0; iBand < this.nBands; iBand++) {
        		for(int line = 0; line < height; line ++)
            		for(int col = 0; col < width; col ++)
            			byteBuf[iBand][line][col] = (byte)noDataValue;	
    		}
        } else if ((dataType == TYPE_SHORT) | (dataType == TYPE_USHORT)) {
        	for (int iBand = 0; iBand < this.nBands; iBand++) {
        		for(int line = 0; line < height; line ++)
            		for(int col = 0; col < width; col ++)
            			shortBuf[iBand][line][col] = (short)noDataValue;	
    		}
        } else if (dataType == TYPE_INT) {
        	for (int iBand = 0; iBand < this.nBands; iBand++) {
        		for(int line = 0; line < height; line ++)
            		for(int col = 0; col < width; col ++)
            			intBuf[iBand][line][col] = (int)noDataValue;	
    		}
        } else if (dataType == TYPE_FLOAT) {
        	for (int iBand = 0; iBand < this.nBands; iBand++) {
        		for(int line = 0; line < height; line ++)
            		for(int col = 0; col < width; col ++)
            			floatBuf[iBand][line][col] = (float)noDataValue;	
    		}
        } else if (dataType == TYPE_DOUBLE) {
        	for (int iBand = 0; iBand < this.nBands; iBand++) {
        		for(int line = 0; line < height; line ++)
            		for(int col = 0; col < width; col ++)
            			doubleBuf[iBand][line][col] = noDataValue;	
    		}
        }
    	
    }
}
