/*
 * Cresques Mapping Suite. Graphic Library for constructing mapping applications.
 *
 * Copyright (C) 2004-5.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 * cresques@gmail.com
 */
package org.cresques.io.datastruct;

import org.cresques.io.data.RasterBuf;
import org.cresques.util.Utilities;

import es.gva.cit.jgdal.GdalColorEntry;
import es.gva.cit.jgdal.GdalColorTable;
import es.gva.cit.jgdal.GdalException;

/**
 * Paleta para raster. Esta consta de los valores RGB de la paleta que son almacenados en un vector de
 * enteros donde cada elemento entero contiene en su interior el RGB completo y del vector de rangos. 
 * Dependiendo de si el tipo de rango es entero o decimal este estar almacenado en un vector de rangos
 * entero (intRange) o enun vector de rangos double (doubleRange). El tipo de dato del rango quedar 
 * almacenado en la variable type.  
 * @author Nacho Brodin (brodin_ign@gva.es)
 */
public class Palette{
	/**
	 * Tipo de dato de la paleta
	 * <UL>
	 * <LI>TYPE_INT = Valido para byte, short e int</LI>
	 * <LI>TYPE_DOUBLE = Valido para float y double</LI>
	 * </UL>
	 */
	private int type = RasterBuf.TYPE_UNDEFINED;
	/**
	 * Lista de rangos para paletas enteras
	 */
	private int[] 		intRange = null;
	/**
	 * Lista de rangos para paletas decimales
	 */
	private double[] 	doubleRange = null;
	/**
	 * Lista de valores RGB
	 */
	private int[]		palette = null;
	/**
	 * Heuristica para paletas enteras. Esta heuristica simple consiste en el valor
	 * intermedio de la tabla, de esta forma reduce la tabla a la mitad.
	 */
	private int 		intHeuristica = Integer.MAX_VALUE;
	/**
	 * Heuristica para paletas decimales. Esta heuristica simple consiste en el valor
	 * intermedio de la tabla, de esta forma reduce la tabla a la mitad.
	 */
	private double 		doubleHeuristica = Double.MAX_VALUE;
	/**
	 * Nombre de la clase asociada a la entrada
	 */
	private String[]	nameClass = null;
	
	private String 		name = null;
	
	public Palette(){}
	
	/**
	 * Constructor. Asigna el nombre de la paleta.
	 * @param name
	 */
	public Palette(String name){
		this.name = name;
	}
	
	/**
	 * Asigna el nombre de la paleta
	 * @param Nombre de la paleta
	 */
	public void addName(String name){
		this.name = name;
	}
	
	/**
	 * Obtiene el nombre de la paleta
	 * @return Nombre de la paleta
	 */
	public String getName(){
		return name;
	}
	
	/**
	 * Obtiene el valor RGB interpolado para un clave entera pasada por parmetro
	 * @param value clave de la cual se quiere obtener el valor RGB de la paleta
	 * @return valor RGB
	 */
	public int getInterpolateRGB2(double value){
		int maxGrad = 127;
		int init = 1;
		int color = 0;
		double beginValue = 0;
		double nextValue = 0;
		for(int i = init; i <= doubleRange.length; i++){
			if(i < doubleRange.length){
				if(value > doubleRange[i]){
					color = palette[i - 1];
					beginValue = doubleRange[i - 1];
					nextValue = doubleRange[i];
					break;
				}
			}else{
				color = palette[i - 1];
				beginValue = doubleRange[i - 1];
				nextValue = 0;
				break;
			}
		}
		int[] colorBegin = Utilities.getARGBFromIntToIntArray(color);
		
		int max = Math.max(Math.max(colorBegin[0], colorBegin[1]), colorBegin[2]);
		
		double distance = Math.abs(beginValue - nextValue);
		double absValue = Math.abs(value - nextValue);
		
		int newBand = 0;
		if(distance != 0)
			newBand = (int)Math.round((absValue * maxGrad) / distance);
		
		int[] res = new int[3];
		
		if((max + maxGrad) > 255){
			for(int i = 0; i < 3; i++)
				if(colorBegin[i] == max){
					res[i] = max - newBand;
					break;
				}else
					res[i] = colorBegin[i];
		}else{
			for(int i = 0; i < 3; i++)
				if(colorBegin[i] == max){
					res[i] = max + newBand;
					break;
				}else
					res[i] = colorBegin[i];
		}
		
		return Utilities.getIntFromARGB(colorBegin[3], res[0], res[1], res[2]);
	}
	
	/**
	 * Obtiene el valor RGB interpolado para un clave entera pasada por parmetro
	 * @param value clave de la cual se quiere obtener el valor RGB de la paleta
	 * @return valor RGB
	 */
	public int getInterpolateRGB2(int value){
		int maxGrad = 127;
		int init = 1;
		int color = 0;
		int beginValue = 0;
		int nextValue = 0;
		for(int i = init; i <= intRange.length; i++){
			if(i < intRange.length){
				if(value > intRange[i]){
					color = palette[i - 1];
					beginValue = intRange[i - 1];
					nextValue = intRange[i];
					break;
				}
			}else{
				color = palette[i - 1];
				beginValue = intRange[i - 1];
				nextValue = 0;
				break;
			}
		}
		int[] colorBegin = Utilities.getARGBFromIntToIntArray(color);
		
		int max = Math.max(Math.max(colorBegin[0], colorBegin[1]), colorBegin[2]);
		
		int distance = Math.abs(beginValue - nextValue);
		int absValue = Math.abs(value - nextValue);
		
		int newBand = 0;
		if(distance != 0)
			newBand = (absValue * maxGrad) / distance;
		
		int[] res = new int[3];
		
		if((max + maxGrad) > 255){
			for(int i = 0; i < 3; i++)
				if(colorBegin[i] == max){
					res[i] = max - newBand;
					break;
				}else
					res[i] = colorBegin[i];
		}else{
			for(int i = 0; i < 3; i++)
				if(colorBegin[i] == max){
					res[i] = max + newBand;
					break;
				}else
					res[i] = colorBegin[i];
		}
		
		return Utilities.getIntFromARGB(colorBegin[3], res[0], res[1], res[2]);
	}
	
	
	/**
	 * Obtiene el valor RGB interpolado para un clave entera pasada por parmetro
	 * @param value clave de la cual se quiere obtener el valor RGB de la paleta
	 * @return valor RGB
	 */
	public int getInterpolateRGB(int value){
		int init = 1;
		int color = 0;
		int beginValue = 0;
		int nextValue = 0;
		int nextColor = 0;
		for(int i = init; i <= intRange.length; i++){
			if(i < intRange.length){
				if(value > intRange[i]){
					color = palette[i - 1];
					beginValue = intRange[i - 1];
					nextValue = intRange[i];
					nextColor = palette[i];
					break;
				}
			}else{
				color = palette[i - 1];
				beginValue = intRange[i - 1];
				nextValue = intRange[0];
				nextColor = palette[0];
				break;
			}
		}
		int[] colorBegin = Utilities.getARGBFromIntToIntArray(color);
		int[] colorEnd = Utilities.getARGBFromIntToIntArray(nextColor);
		
		int dif = Math.abs(beginValue - nextValue);
		double difR = Math.abs(colorBegin[0] - colorEnd[0]) / (double)dif;
		double difG = Math.abs(colorBegin[1] - colorEnd[1]) / (double)dif;
		double difB = Math.abs(colorBegin[2] - colorEnd[2]) / (double)dif;
		
		int[] res = new int[3];
		res[0] = (int)(colorBegin[0] + difR * (value - colorBegin[0]));
		res[1] = (int)(colorBegin[1] + difG * (value - colorBegin[1]));
		res[2] = (int)(colorBegin[2] + difB * (value - colorBegin[2]));
		
		return Utilities.getIntFromARGB(colorBegin[3], res[0], res[1], res[2]);
	}
	
	/**
	 * Obtiene el valor RGB para un clave entera pasada por parmetro
	 * @param value clave de la cual se quiere obtener el valor RGB de la paleta
	 * @return valor RGB
	 */
	public int getRGB(int value){
		int init = 1;
		/*if(value > intHeuristica)
			init = intRange.length >> 1;*/
		for(int i = init; i <= intRange.length; i++)
			if(i < intRange.length){
				if(value > intRange[i])
					return palette[i - 1];
			}else{
				return palette[i - 1];
			}
		return 0;
	}
	
	/**
	 * Obtiene el valor RGB para un clave decimal pasada por parmetro
	 * @param value clave de la cual se quiere obtener el valor RGB de la paleta
	 * @return valor RGB
	 */
	public int getRGB(double value){
		int init = 1;
		/*if(value > doubleHeuristica)
			init = doubleRange.length >> 1;*/
		for(int i = init; i <= doubleRange.length; i++)
			if(i < doubleRange.length){
				if(value > doubleRange[i])
					return palette[i - 1];
			}else{
				return palette[i - 1];
			}
		return 0;
	}
	
	/**
	 * Genera una paleta a partir de una tabla de entradas de la forma
	 * <UL>
	 * <LI>(String) Nombre de la clase</LI>
	 * <LI>(Integer) Rojo</LI>
	 * <LI>(Integer) Verde</LI>
	 * <LI>(Integer) Azul</LI>
	 * <LI>(Double) Valor</LI>
	 * </UL>
	 * @param entries
	 */
	public void createPaletteFromTable(Object[][] entries, int type){
		this.type = type;
		nameClass = new String[entries.length];
		palette = new int[entries.length];
		if(type <= RasterBuf.TYPE_INT){
			intRange = new int[entries.length];
			intHeuristica = (int)((Double)entries[(entries.length >> 1)][4]).doubleValue();
		}else{
			doubleRange = new double[entries.length];
			doubleHeuristica = ((Double)entries[(entries.length >> 1)][4]).doubleValue();
		}
		for(int entry = 0; entry < entries.length; entry++){
			//palette[entry] = 0xff000000;
			for(int value = 0; value < entries[entry].length; value++){
				switch(value){
				case 0: nameClass[entry] = (String)entries[entry][value];
						break;
				case 1: palette[entry] |= ((((Integer)entries[entry][value]).intValue() & 0x000000ff) << 16);
						break;
				case 2: palette[entry] |= ((((Integer)entries[entry][value]).intValue() & 0x000000ff) << 8);
						break;
				case 3: palette[entry] |= (((Integer)entries[entry][value]).intValue() & 0x000000ff);
						break;
				case 4: if(type <= RasterBuf.TYPE_INT)
							intRange[entry] = ((Double)entries[entry][value]).intValue();
						else
							doubleRange[entry] = ((Double)entries[entry][value]).doubleValue();
						break;
				case 5: palette[entry] |= ((((Integer)entries[entry][value]).intValue() & 0x000000ff) << 24);
						break;
				}
			}
		}
	}
	
	/**
	 * Crea una paleta a partir de un objeto GdalColorTable. Esto es necesario para los ficheros
	 * que tienen un paleta asignada, como los gif, y que son tratados por Gdal. Se pasa la tabla
	 * de color leda desde gdal y se crea directamente un objeto Palette. 
	 * @param table
	 */
	public void createPaletteFromGdalColorTable(GdalColorTable table){
		try{
			type = RasterBuf.TYPE_BYTE;
			nameClass = new String[table.getColorEntryCount()];
			palette = new int[table.getColorEntryCount()];
			intRange = new int[table.getColorEntryCount()];
			
			int cont = table.getColorEntryCount() - 1;
	    	for(int iEntry = 0; iEntry < table.getColorEntryCount(); iEntry++){
	    		GdalColorEntry entry = table.getColorEntryAsRGB(iEntry);
	    		nameClass[cont] = "";
	    		palette[cont] = 0x00000000;
	    		palette[cont] |= ((entry.c4 & 0x000000ff) << 24);
	    		palette[cont] |= ((entry.c1 & 0x000000ff) << 16);
	    		palette[cont] |= ((entry.c2 & 0x000000ff) << 8);
	    		palette[cont] |= (entry.c3 & 0x000000ff);
	    		intRange[cont] = iEntry;
	    		cont --;
	    	}
		}catch(GdalException ex){
			//No se crea la paleta
		}
	}
	
	/**
	 * Carga una paleta desde un fichero XML
	 * @param file nombre del fichero
	 * @param palette Nombre de la paleta
	 */
	public void loadPaletteFromXML(String file, String palette){
		//TODO: Implemetar cargar paleta desde el directorio gvSIG
	}
	
	/**
	 * Carga una paleta desde el fichero .rmf asociado a la imagen
	 * @param file nombre del fichero de imagen
	 */
	public void loadPaletteFromRMF(String file){
		//TODO: Implemetar cargar paleta desde .rmf
	}
	
	/**
	 * Salva una paleta desde al fichero .rmf asociado a la imagen
	 * @param file nombre del fichero de imagen
	 */
	public void savePaletteToRMF(String file){
		//TODO: Implemetar salvar paleta a .rmf
	}

	/**
	 * Obtiene la paleta
	 * @return Paleta
	 */
	public int[] getPalette() {
		return palette;
	}

	/**
	 * Asigna una paleta
	 * @param palette Paleta
	 */
	public void setPalette(int[] palette) {
		this.palette = palette;
	}

	/**
	 * Obtiene el tipo del rango de la paleta que corresponde con los tipos de rasterBuf
	 * @return Tipo de rango, entero o double
	 */
	public int getType() {
		return type;
	}
	
	/**
	 * Asigna el tipo del rango de la paleta que corresponde con los tipos de rasterBuf
	 * @param Tipo de rango, entero o double
	 */
	public void setType(int type) {
		this.type = type;
	}

	/**
	 * Asigna los rangos si el tipo es decimal
	 * @param rangos
	 */
	public void setDoubleRange(double[] value) {
		doubleRange = value;
	}

	/**
	 * Asigna los rangos si el tipo es entero
	 * @param values
	 */
	public void setIntRange(int[] values) {
		intRange = values;
	}

	/**
	 * Obtiene los rangos si el tipo es decimal
	 * @return rangos
	 */
	public double[] getDoubleRange() {
		return doubleRange;
	}

	/**
	 * Obtiene los rangos si el tipo es entero
	 * @return rangos
	 */
	public int[] getIntRange() {
		return intRange;
	}

	
	/**
	 * Obtiene los nombres de las clases de la paleta
	 * @return Array de cadenas. Cada una corresponde con un nombre de clase
	 * que corresponde a cada rango de tipos.
	 */
	public String[] getNameClass() {
		return nameClass;
	}
	
	/**
	 * Asigna los nombres de las clases de la paleta
	 * @param names Array de cadenas. Cada una corresponde con un nombre de clase
	 * que corresponde a cada rango de tipos.
	 */
	public void setNameClass(String[] names) {
		nameClass = names;
	}
	
	
}
