/*
 * Created on 01-sep-2006
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
*
* $Id: FLayerStatus.java 13980 2007-09-24 12:03:13Z ppiqueras $
* $Log$
* Revision 1.1.2.4  2007-09-24 12:03:13  ppiqueras
* Translated comments to English language and add new ones.
*
* Revision 1.1.2.3  2007/02/21 08:57:28  fjp
* Bug con el driver loaded corregido. Afectaba a los eventos de capas
*
* Revision 1.1.2.2  2007/02/20 08:51:20  fjp
* Cambios para que el driver avise de cundo a terminado de trabajar
*
* Revision 1.1.2.1  2006/11/15 04:11:12  jjdelcerro
* *** empty log message ***
*
* Revision 1.1  2006/09/21 17:23:39  azabala
* First version in cvs
*
*
*/
package com.iver.cit.gvsig.fmap.layers;

import java.util.ArrayList;
import java.util.List;

import com.iver.cit.gvsig.fmap.DriverException;

/**
 * <p>All <i>fmap</i> layers can be in a set of states, that their combination represent the status of a layer.</p>
 * 
 * <p>The <code>FLayerStatus</code> class supports the common status for all kind of {@link FLyrDefault FLyrDefault} layers.</p>
 * 
 * <p>This means that not necessarily all layers can stay in all possible status, it depends on its particular
 * constitution.</p>
 * 
 * <p>Layer status states defined:
 *  <ul>
 *  <li><i><b>Available</b></i>: the layer is available if the source of its data is on-line.</li>
 *  <li><i><b>Visible</b></i>: one layer is visible if it's selected its <i>check box</i> associated in TOC. This implies that 
 *  layer will tried to be painted unless the necessary data would be unavailable; this situation is more frequent with layers of
 *  remote services.</li>
 *  <li><i><b>Active</b></i>: the layer is active if it's selected at the TOC.</li>
 *  <li><i><b>Dirty</b></i>: one layer is dirty if needs to be refreshed.</li>
 *  <li><i><b>In TOC</b></i>: the layer is in TOC if it's at the view's TOC's tree of layers.</li>
 *  <li><i><b>Editing</b></i>: if the layer it's being edited now.</li>
 *  <li><i><b>Writable</b></i>: if can be modified and saved the changes.</li>
 *  <li><i><b>Cache drawn layers</b></i>: this state reports if the layer stores previous draws. This is useful to accelerate the draw of the layer.</li>
 *  <li><i><b>Driver loaded</b></i>: this state reports if the driver for managing the layer has been loaded successfully.</li>
 *  </ul>
 * </p>
 * 
 * @see IFLayerStatus
 * 
 * @author azabala
 */
public class FLayerStatus implements IFLayerStatus {
	/**
	 * <p>Layer's availability: the layer is available if the source of its data is on-line.</p>
	 */
	public boolean available;
	
	/**
	 * <p>Layer's visibility: the layer is visible if it's selected its <i>check box</i> associated in TOC. This implies that 
	 *  layer will tried to be painted unless the necessary data would be unavailable; this situation is more frequent with layers of
	 *  remote services.</p>
	 */
	public boolean visible;
	
	/**
	 * <p>The layer is active if it's selected at the TOC.</p>
	 */
	public boolean active;
	
	/**
	 * <p>One layer is dirty if needs to be refreshed.</p>
	 */
	public boolean dirty;
	
	/**
	 * <p>The layer is in TOC if it's at the view's TOC's tree of layers.</p>
	 */
	public boolean inTOC;
	
	/**
	 * <p>This parameter reports if the layer it's being edited now.</p>
	 */
	public boolean editing;
	
	/**
	 * <p>This parameter reports if the layer can be modified and saved the changes.</p>
	 */
	public boolean writable;
	
	/**
	 * <p>This state reports if the layer stores previous draws. That's useful to accelerate the draw of the layer.</p>
	 */
	public boolean cacheDrawnLayers;
	
	/**
	 * <p>This state reports if the driver for managing the layer has been loaded successfully.</p>
	 */
	private boolean driverLoaded;
	
	/**
	 * 
	 */
	public boolean enabled;
	
	/**
	 * <p>List with the information of the driver errors produced working this layer.</p>
	 */
	private ArrayList layerErrors;

	/**
	 * <p>Creates an instance with the default status of a {@link FLyrDefault FLyrDefault} layer:
	 *  <ul>
	 *  <li><b><i>Visible</i></b>: <code>true</code> .</li>
	 *  <li><b><i>Active</i></b>: <code>false</code> .</li>
	 *  <li><b><i>In TOC</i></b>: <code>true</code> .</li>
	 *  <li><b><i>Editing</i></b>: <code>false</code> .</li>
	 *  <li><b><i>Cache drawn layers</i></b>: <code>false</code> .</li>
	 *  <li><b><i>Dirty</i></b>: <code>false</code> .</li>
	 *  <li><b><i>Available</i></b>: <code>true</code> .</li>
	 *  <li><b><i>Driver loaded</i></b>: <code>true</code> .</li>
	 *  <li><b><i>Writable</i></b>: <code>false</code> .</li>
	 *  <li>Without driver errors.</li>
	 *  </ul>
	 * </p>
	 */
	public FLayerStatus(){
		this.layerErrors = new ArrayList();
		visible = true;
		active = false;
		inTOC = true;
		editing = false;
		cacheDrawnLayers = false;
		dirty = false;
		available = true;
		driverLoaded = true;
		writable = false;
		
	}

	/**
	 * <p>Stores information of an exception produced working with a driver of the layer.</p>
	 * 
	 * @param error the driver exception
	 */
	public void addLayerError(DriverException error){
		layerErrors.add(error);
	}

	/**
	 * <p>Gets the number of driver exceptions produced working with the layer.</p>
	 * 
	 * @return number of driver exceptions produced
	 */
	public int getNumErrors(){
		return layerErrors.size();
	}

	/**
	 * <p>Gets the information of the <i>nth</i> layer driver exception registered in the status.</p>
	 * 
	 * @param i ith layer exception registered
	 * 
	 * @return the <i>nth</i> layer driver exception registered
	 */
	public DriverException getError(int i){
		return (DriverException) layerErrors.get(i);
	}

	/**
	 * <p>Returns if there have been driver errors working with the layer.</p>
	 * 
	 * @return <code>true</code> if there have driver errors working with the layer; <code>false</code> otherwise
	 */
	public boolean isOk(){
		return layerErrors.size() == 0;
	}

	/**
	 * <p>Returns a list with the driver errors produced in a layer.</p>
	 * 
	 * @return list with the driver errors produced in a layer
	 */
	public List getErrors() {
		return layerErrors;
	}

	/**
	 * <p>Returns if has been loaded successfully the driver for managing the layer.</p>
	 * 
	 * @return <code>true</code> if has been loaded successfully the driver for managing the layer; <code>false</code> otherwise
	 */
	public boolean isDriverLoaded() {
		return driverLoaded;
	}

	/**
	 * <p>Sets if has been loaded successfully the driver for managing the layer.</p>
	 * 
	 * @param driverLoaded <code>true</code> if has been loaded successfully the driver for managing the layer; <code>false</code> otherwise
	 */
	public void setDriverLoaded(boolean driverLoaded) {
		this.driverLoaded = driverLoaded;
	}
}