/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.layers.layerOperations;

import com.iver.cit.gvsig.fmap.layers.CancelationException;
import com.iver.cit.gvsig.fmap.layers.FLayer;
import com.iver.cit.gvsig.fmap.layers.LayerCollectionListener;

/**
 * <p>Interface with methods of a collection of layers.</p> 
 */
public interface LayerCollection {
	/**
	 * Adds a listener about the events produced on a layers collection.
	 *
	 * @param listener a <code>LayerCollectionListener</code>
	 * 
	 * @see #removeLayerCollectionListener(LayerCollectionListener)
	 */
	public void addLayerCollectionListener(LayerCollectionListener listener);

	/**
	 * Removes a listener about the events produced on a layers collection.
	 *
	 * @param listener a <code>LayerCollectionListener</code>
	 * 
	 * @see #addLayerCollectionListener(LayerCollectionListener)
	 */
	public void removeLayerCollectionListener(LayerCollectionListener listener);

	/**
	 * Adds a layer at the end of the layers list.
	 *
	 * @param layer a layer
	 *
	 * @throws CancelationException any exception produced during the cancellation of the driver.
	 * 
	 * @see #removeLayer(FLayer)
	 * @see #removeLayer(int)
	 * @see #removeLayer(String)
	 */
	public void addLayer(FLayer layer) throws CancelationException;

	/**
	 * <p>Moves a layer of the collection to another position in internal list. It does not consider subnodes.</p>
	 *
	 * @param from origin position
	 * @param to destination position
	 *
	 * @throws CancelationException any exception produced during the cancellation of the driver.
	 */
	public void moveTo(int from, int to) throws CancelationException;

	/**
	 * Removes a layer from the collection.
	 *
	 * @param lyr a layer
	 *
	 * @throws CancelationException any exception produced during the cancellation of the driver.
	 * 
	 * @see #removeLayer(int)
	 * @see #removeLayer(String)
	 * @see #addLayer(FLayer)
	 */
	public void removeLayer(FLayer lyr) throws CancelationException;

	/**
	 * Removes a layer using its identifier.
	 *
	 * @param idLayer layer id
	 * 
	 * @see #removeLayer(FLayer)
	 * @see #removeLayer(String)
	 * @see #addLayer(FLayer)
	 */
	public void removeLayer(int idLayer);

	/**
	 * Removes a layer using its name.
	 *
	 * @param layerName the name of the layer
	 * 
	 * @see #removeLayer(FLayer)
	 * @see #removeLayer(int)
	 * @see #addLayer(FLayer)
	 */
	public void removeLayer(String layerName);

	/**
	 * Returns an array with all visible layers in this collection.
	 *
	 * @return array with layers, or <code>null</code> if no there is no layer visible
	 * 
	 * @see #setAllVisibles(boolean)
	 */
	public FLayer[] getVisibles();

	/**
	 * Returns an array with all active layers in this collection.
	 *
	 * @return array with layers, or <code>null</code> if no there is no layer active
	 * 
	 * @see #setAllActives(boolean)
	 */
	public FLayer[] getActives();

	/**
	 * Returns the ith-output directed son (from bottom up) of this collection.
	 *
	 * @param index index of the ith-output layer in this collection.
	 *
	 * @return a layer
	 * 
	 * @see #getLayer(String)
	 */
	public FLayer getLayer(int index);

	/**
	 * Returns a directed son layer of this collection, using its name.
	 *
	 * @param layerName name of a layer of this collection
	 *
	 * @return a layer
	 * 
	 * @see #getLayer(int)
	 */
	public FLayer getLayer(String layerName);

	/**
	 * <p>Returns the number of layers that are at the same level as this one.</p>
	 * 
	 * <p>Doesn't counts the sublayers of a <code>FLayers</code> node object.</p>
	 *
	 * @return number >= 0 with layers that are at the same level
	 * 
	 * @see #getLayer(int)
	 * @see #getLayer(String)
	 */
	public int getLayersCount();

	/**
	 * Changes the status of all layers to active or inactive.
	 *
	 * @param active the boolean to be set
	 * 
	 * @see #getActives()
	 */
	public void setAllActives(boolean active);

	/**
	 * Changes the status of all layers to visible or invisible.
	 *
	 * @param visible the boolean to be set
	 * 
	 * @see #getVisibles()
	 */
	public void setAllVisibles(boolean visible);
}
