/*
 * Created on 19-sep-2005
 *
 * gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
package com.iver.cit.gvsig.fmap.rendering;

import java.awt.Graphics2D;

import com.iver.cit.gvsig.fmap.ViewPort;
import com.iver.cit.gvsig.fmap.core.IGeometry;
import com.iver.cit.gvsig.fmap.core.ISymbol;

/** 
 * <p>The class <code>FGraphic</code> defines a graphic symbol according to the <i>FMap philosophy</i>.</p>
 * 
 * <p>This symbol represents a drawingable geometry with an identifier, and can optionally have attached a tag
 *  that can be text or an object.</p> 
 */
public class FGraphic {
	/**
	 * Identifies this graphic symbol.
	 * 
	 * @see #getIdSymbol()
	 * @see #setIdSymbol(int)
	 */
    int idSymbol;
    
    /**
     * Geometry of this graphic symbol.
     * 
     * @see #getGeom()
     * @see #setGeom(IGeometry)
     */
    IGeometry geom;
    
    /**
     * Optional text associated to this graphic symbol as a tag.
     * 
     * @see #getTag()
     * @see #setTag(String)
     */
    String tag;
    
    /**
     * Optional object <i>(example: an icon)</i> associated to this graphic symbol as a tag.
     * 
     * @see #getObjectTag()
     * @see #setObjectTag(Object)
     */
    Object objectTag;

    /**
     * <p>Creates a graphic object according to the <i>FMap philosophy</i>, from a geometry and an identifier.</p> 
     *
     * @param geom the geometry that this symbol will represent
     * @param idSymbol the identifier
     */
    public FGraphic(IGeometry geom, int idSymbol)
    {
        this.idSymbol = idSymbol;
        this.geom = geom;
    }

    /**
     * Gets the text tag associated to this symbol.
     * 
     * @return the text tag associated to this symbol, or <code>null</code> if undefined
     * 
     * @see #setTag(String)
     */
    public String getTag() {
        return tag;
    }

    /**
     * Sets a text tag associated to this symbol.
     * 
     * @param tag a text tag associated to this symbol
     * 
     * @see #getTag()
     */
    public void setTag(String tag) {
        this.tag = tag;
    }

    /**
     * Gets the geometry of this symbol.
     * 
     * @return an object indicating this component's geometry
     * 
     * @see #setGeom(IGeometry)
     */
    public IGeometry getGeom() {
        return geom;
    }

    /**
     * Changes the geometry of this symbol to the new one.
     * 
     * @param geom an object indicating this component's geometry
     * 
     * @see #getGeom()
     */
    public void setGeom(IGeometry geom) {
        this.geom = geom;
    }

    /**
     * Gets the identifier of this symbol.
     * 
     * @return Returns the idSymbol.
     * 
     * @see #setIdSymbol(int)
     */
    public int getIdSymbol() {
        return idSymbol;
    }

    /**
     * Changes the identifier of this symbol to the new one.
     *     
     * @param idSymbol the new identifier
     * 
     * @see #getIdSymbol()
     */
    public void setIdSymbol(int idSymbol) {
        this.idSymbol = idSymbol;
    }

    /**
     * <p>Defines the default logic of drawing an <i>FMap</i> graphic symbol.</p>
     * 
     * <p>Using the rendering defined in {@link Graphics2D Graphics2D}, the symbol will
     *  be drawn in the viewport. That symbol must be which has the id <code>idSymbol<code>
     *  if we want to draw which represents this class.</p>
     * 
     * @param g for rendering 2-dimensional shapes, text and images on the Java(tm) platform
     * @param viewPort information for drawing this graphic item
     * @param theSymbol the symbol that will be drawn (this parameter is generally used indicating
     *  the object that has the identifier <code>idSymbol</code>) 
     */
    public void draw(Graphics2D g, ViewPort viewPort, ISymbol theSymbol )
    {
        if (theSymbol.isShapeVisible())
        {
            IGeometry cloneGeom = geom.cloneGeometry();
            cloneGeom.draw(g,viewPort,theSymbol);
        }
    }

    /**
     * Gets the object tag associated to this symbol.
     * 
     * @return the object tag associated to this symbol, or <code>null</code> if  undefined
     * 
     * @see #setObjectTag(Object)
     */
	public Object getObjectTag() {
		return objectTag;
	}

	/**
	 * Sets an object tag associated to this symbol.
	 * 
	 * @param objectTag an object tag associated to this symbol
	 * 
	 * @see #getObjectTag()
	 */
	public void setObjectTag(Object objectTag) {
		this.objectTag = objectTag;
	}
}
