package org.gvsig.gpe.gml.parser.v2.geometries;

import java.io.IOException;

import org.gvsig.gpe.gml.GMLTags;
import org.gvsig.gpe.gml.GPEDefaultGmlParser;
import org.gvsig.gpe.gml.utils.CompareUtils;
import org.gvsig.gpe.gml.utils.CoordsContainer;
import org.gvsig.gpe.warnings.PolygonNotClosedWarning;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id:LinearRingTypeBinding.java 195 2007-11-26 09:02:22Z jpiera $
 * $Log$
 * Revision 1.4  2007/05/16 09:29:12  jorpiell
 * The polygons has to be closed
 *
 * Revision 1.3  2007/05/15 11:55:11  jorpiell
 * MultiGeometry is now supported
 *
 * Revision 1.2  2007/05/14 09:31:06  jorpiell
 * Add the a new class to compare tags
 *
 * Revision 1.1  2007/05/07 12:58:42  jorpiell
 * Add some methods to manage the multigeometries
 *
 *
 */
/**
 * It parses a gml:linearRingType object. Example:
 * <p>
 * <pre>
 * <code>
 * &lt;LinearRing&gt;
 * &lt;coordinates&gt;0.0,0.0 100.0,0.0 50.0,100.0 0.0,0.0&lt;/coordinates&gt;
 * &lt;/LinearRing&gt;
 * </code>
 * </pre>
 * </p> 
 * @author Jorge Piera LLodr (jorge.piera@iver.es)
 */
public class LinearRingTypeBinding extends GeometryBinding{
	protected CoordsContainer coordsContainer = null;
	
	/**
	 * It parses the gml:LinearRing tag and return the 
	 * Object
	 * @param parser
	 * The XML parser
	 * @param handler
	 * The GPE parser that contains the content handler and
	 * the error handler
	 * @return
	 * A linear ring
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	public Object parse(XmlPullParser parser,GPEDefaultGmlParser handler) throws XmlPullParserException, IOException{
		Object linearRing = null;
		coordsContainer = null;
		
		super.setAtributtes(parser, handler.getErrorHandler());

		double[][] coordinates;
		
		coordinates = parseCoordinates(parser, handler);	
		
		linearRing = handler.getContentHandler().startLinearRing(id, coordinates[0], coordinates[1], coordinates[2], srsName);
		handler.getContentHandler().endLinearRing(linearRing);
		
		return linearRing;
	}
	
	/**
	 * It parses the gml:LinearRing tag and return the 
	 * coordinates
	 * @param parser
	 * The XML parser
	 * @param handler
	 * The GPE parser that contains the content handler and
	 * the error handler
	 * @return
	 * An array of coordinates
	 * @throws XmlPullParserException
	 * @throws IOException
	 * @throws PolygonNotClosedWarning 
	 */
	public double[][] parseCoordinates(XmlPullParser parser,GPEDefaultGmlParser handler) throws XmlPullParserException, IOException {
		boolean endFeature = false;
		int currentTag;
		coordsContainer = null;
		double[][] coordinates = null;


		String tag = parser.getName();
		currentTag = parser.getEventType();

		while (!endFeature){
			switch(currentTag){
			case XmlPullParser.START_TAG:
				coordinates = parseTag_(parser, handler, tag);
				break;
			case XmlPullParser.END_TAG:
				if (CompareUtils.compareWithOutNamespace(tag,GMLTags.GML_LINEARRING)){						
					endFeature = true;
					//If the file contains the COORD tag instead of the
					//coordinates
					if (coordsContainer != null){
						coordinates = coordsContainer.getCoordinates();									
					}					
				}
				break;
			case XmlPullParser.TEXT:					

				break;
			}
			if (!endFeature){					
				currentTag = parser.next();
				tag = parser.getName();
			}
		}			
		//The first and the last coordinates has to be the same
		if ((coordinates[0][0] != coordinates[0][coordinates[0].length - 1]) ||
				(coordinates[1][0] != coordinates[1][coordinates[1].length - 1]) ||
				(coordinates[2][0] != coordinates[2][coordinates[2].length - 1])){
			handler.getErrorHandler().addWarning(new PolygonNotClosedWarning(coordinates));
		}
		return coordinates;
	}
	
	/**
	 * 
	 * @param parser
	 * @param handler
	 * @param tag
	 * @param coordsContainer
	 * @return
	 * @throws XmlPullParserException
	 * @throws IOException
	 */
	protected double[][] parseTag_(XmlPullParser parser,GPEDefaultGmlParser handler, String tag) throws XmlPullParserException, IOException{
		double[][] coordinates = null;
		if (CompareUtils.compareWithOutNamespace(tag,GMLTags.GML_COORDINATES)){
			coordinates = handler.getProfile().getCoordinatesTypeBinding().
			parse(parser, handler);
		}else if (CompareUtils.compareWithOutNamespace(tag,GMLTags.GML_COORD)){
			double[] coord = handler.getProfile().getCoordTypeBinding().
			parse(parser, handler);
			if (coordsContainer == null){
				coordsContainer = new CoordsContainer();
			}
			coordsContainer.addCoordinates(coord);
		}
		return coordinates;
	}
	
	
}
