package org.gvsig.remoteClient.gml;

import java.awt.geom.Rectangle2D;

import org.gvsig.remoteClient.gml.exceptions.GMLException;
import org.gvsig.remoteClient.gml.factories.IGeometriesFactory;
import org.gvsig.remoteClient.gml.schemas.XMLSchemaParser;
import org.kxml2.io.KXmlParser;

/* gvSIG. Sistema de Informacin Geogrfica de la Generalitat Valenciana
 *
 * Copyright (C) 2004 IVER T.I. and Generalitat Valenciana.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,USA.
 *
 * For more information, contact:
 *
 *  Generalitat Valenciana
 *   Conselleria d'Infraestructures i Transport
 *   Av. Blasco Ibez, 50
 *   46010 VALENCIA
 *   SPAIN
 *
 *      +34 963862235
 *   gvsig@gva.es
 *      www.gvsig.gva.es
 *
 *    or
 *
 *   IVER T.I. S.A
 *   Salamanca 50
 *   46005 Valencia
 *   Spain
 *
 *   +34 963163400
 *   dac@iver.es
 */
/* CVS MESSAGES:
 *
 * $Id: GMLFeaturesParser.java 18271 2008-01-24 09:06:43Z jpiera $
 * $Log$
 * Revision 1.3  2007-01-15 13:11:00  csanchez
 * Sistema de Warnings y Excepciones adaptado a BasicException
 *
 * Revision 1.2  2006/12/22 11:25:44  csanchez
 * Nuevo parser GML 2.x para gml's sin esquema
 *
 * Revision 1.1  2006/08/10 12:00:49  jorpiell
 * Primer commit del driver de Gml
 *
 *
 */
/**
 * This class has all the methods that a GML parser
 * must implement. It has methods to parse some data and
 * methods to get info about the GML file
 * 
 * @author Jorge Piera Llodr (piera_jor@gva.es)
 * @author Carlos Snchez Perin (sanchez_carper@gva.es)
 * 
 */
public abstract class GMLFeaturesParser {
	private XMLSchemaParser parser = null;
	private IGeometriesFactory factory = null;
	private String documentRoot = null;	
	private Extent extent = null;
		
	public GMLFeaturesParser(IGeometriesFactory factory,XMLSchemaParser parser) throws Exception {
		super();
		this.factory = factory;
		this.parser = parser;
		initialize();
	}
	
	/**
	 * Gets one iterator to retrieve all the 
	 * GML features
	 * @param file
	 * File to parse
	 * @return
	 */
	public abstract IGMLFeaturesIterator getFeaturesReader() throws GMLException;	
	/**
	 * It initializes the GML file. It parses all the GML code
	 * that is located before the GML features
	 * @throws GMLException
	 */
	private void initialize() throws Exception{
		//It changes the namespace to "gml" for check all the features in the file
		parser.setSchema("gml");
		int currentTag;		
				
		currentTag = parser.getEventType();		
		switch(currentTag){
			case KXmlParser.START_TAG:
				//This is the main tag of the document
				documentRoot = parser.getName();
				break;
			case KXmlParser.END_TAG:				
				break;
			case KXmlParser.TEXT:                   
				break;
		}
		//now it parses the header of the file
		initializeHeader();
	}	
	
	/**
	 * It initializes some GML fields like the BBOX, the imports,
	 * etc. 
	 * @throws GMLException
	 * @throws BaseException 
	 */
	public abstract void initializeHeader() throws GMLException;
		
	/**
	 * @return Returns the extent.
	 */
	public Extent getExtent() {
		return extent;
	}

	/**
	 * @return Returns the documentRoot.
	 */
	public String getDocumentRoot() {
		return documentRoot;
	}

	/**
	 * @return Returns the factory.
	 */
	public IGeometriesFactory getFactory() {
		return factory;
	}

	/**
	 * @return Returns the parser.
	 */
	public XMLSchemaParser getParser() {
		return parser;
	}

	/**
	 * @param extent The extent to set.
	 */
	public void setExtent(Extent extent) {
		this.extent = extent;
	}	
	
	/************************************************************************************
	 * Class <Extent>																	*
	 * This class take the cords of reference where it will draw the shape				*
	 * Ussualy this references are indicated by the <boundedBy> tag.					*
	 * <gml:boundedBy>																	*
	 *	 <gml:Box srsName="http://www.opengis.net/gml/srs/epsg.xml#23030">				*
	 *		<gml:coordinates decimal="." cs="," ts=" ">									*
	 *		-135.15989685058594,43.97196475963302 -41.46491900991063,76.32874354251143	*
	 *		</gml:coordinates>															*
	 *	 </gml:Box>																		*
	 * </gml:boundedBy>																	*
	 * **********************************************************************************/
	
	public static class Extent{
		private Rectangle2D extent = null;
		
		//it has the reference system.
		private String srs = null;
		
		public Extent(Rectangle2D extent, String srs) {
			super();			
			this.extent = extent;
			this.srs = srs;
		}
		/**
		 * @return Returns the extent.
		 */
		public Rectangle2D getExtent() {
			return extent;
		}
		/**
		 * @return Returns the srs.
		 */
		public String getSrs() {
			return srs;
		}
	}
}
